# frozen_string_literal: true

module Hermes
  module Api
    module Dmo
      class EntityCodesController < Hermes::Api::Dmo::ApplicationController
        before_action :authorize_entreprise!, except: [:show, :index, :check_entity_code_validity]

        class << self
          def mutate_params
            {
              code: {type: 'String'},
              expire_at: {type: 'DateTime'},
              entity_id: {type: 'String'},
              applicant_id: {type: 'String'},
            }
          end 
      
          def acceptable_params
            {
            }.merge(self.mutate_params).merge(super)
          end 
        end

        def index
          acceptable_params *self.class.acceptable_params.keys

          paginate(
            index_aggregation(no_acl: true),
            each_serializer: @serializer,
            **@options
          )
        end

        def destroy
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:unauthorized, :not_owned) unless @instance.user_id == @user.id
          return render(json: @instance.errors, status: :unprocessable_entity) unless @instance.destroy

          render(body: nil, status: :no_content)
        end

        def check_entity_code_validity
          params.require(:code)
          code = params[:code]
          entity_code = Hermes::EntityCode.find_by(code: code)
          return code_not_found(code) unless entity_code
          return code_expired(code) if entity_code.expired?

          render json: {
            success: true,
            code: code,
            message: 'Code valide et non expiré.'
          }, status: :ok

        rescue ActionController::ParameterMissing => e
          render json: { success: false, message: e.message }, status: :not_acceptable
        end

        private

        def authorize_entreprise!
          return render_json_error(:unauthorized, :not_entreprise) unless current_user_roles.include?('entreprise') || current_user_roles.include?('superadministrateur')
        end

        def code_not_found(code)
          render json: {
            success: false,
            message: "Le code '#{code}' n'existe pas."
          }, status: :not_found
        end

        def code_expired(code)
          render json: {
            success: false,
            message: "Le code '#{code}' est expiré."
          }, status: :unprocessable_entity
        end
      end
    end
  end
end