# frozen_string_literal: true

# checker Api::Dmo::HermesController avant d'ajouter des methodes
module Hermes
  module Api
    module Dmo
      class HousingApplicationsController < Hermes::Api::Dmo::ApplicationController
        require Rails.root.join('lib', 'core_extentions', 'parameters', 'handle_sort').to_s

        class << self
          def mutate_params
            {
              housing_request_id: { type: 'String'},
              housing_offer_id: { type: 'String'},
              epci: { type: 'Integer'},
              max_date: { type: 'DateTime'},
              status: { type: 'String'},
              visit_status: { type: 'String'},
              visit_conducted_date: { type: 'DateTime'},
              lessor_event_informations: { type: 'String'},
              last_status_update_date: { type: 'Date'},
              rank: { type: 'Integer'},
              date_event: { type: 'Date'},
              motive_code: { type: 'String'},
            }
          end 
      
          def acceptable_params
            {
              applicated_at: { type: DateTime },
              status_updated_at: { type: DateTime },
              visit_conducted_date: { type: DateTime },
              visit_status_updated_at: { type: DateTime },
              profile_score: { type: Integer },
              eligible_position: { type: Integer },
              applicated_position: { type: Integer },
              is_active: { type: Boolean },
              als_id: { type: String },
              cotations: { type: Hash },
              business_cotation: { type: Float },
              applicant_used_for_siret_firstname: { type: String },
              applicant_used_for_siret_lastname: { type: String },
              applicant_used_for_siret_email: { type: String },
              applicant_used_for_siret_birth_date: { type: Date },
            }.merge(self.mutate_params).merge(super)
          end 
      
          def regex_params
            {}
          end 
        end
        
        def index
          acceptable_params *self.class.acceptable_params.keys

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          options = {}

          aggregation = index_aggregation

          if params[:unique_numbers]
            housing_request_ids = Hermes::HousingRequest.where(unique_numbers: params[:unique_numbers]).pluck(:id)
            aggregation = if housing_request_ids.empty?
              aggregation.none
            else
              aggregation.where(:housing_request_id.in => housing_request_ids)
            end
          end 

          options[:join_housing_offer]= true if params[:join_housing_offer]
          options[:lignt_serializer] = true 
          
          paginate(
            aggregation,
            each_serializer: Hermes::HousingApplicationSerializer,
            **options
          )
        end

        def show
          @options[:join_housing_offer]= true if params[:join_housing_offer]
          super
        end

        def create
          create_without_render
          @instance.skip_check = true if params[:options]&.include?('skip_check')
      
          return render_json_error(:unprocessable_entity, :no_right_to_change_status) unless can_update_status
          return respond_with_errors(@instance) unless @instance.save
      
          render(jsonapi: @instance, serializer: @serializer, status: :created)
        end

        def update
          return render_json_error(:unprocessable_entity, :no_right_to_change_status) unless can_update_status
          super
        end 

        def update_status
          return render_json_error(:unauthorized, :no_right_to_change_status) unless is_admin
          {
            status: :missing_status,
            unique_number: :missing_unique_number,
            external_ref: :missing_external_ref
          }.each do |field, error_key|
            return render_json_error(:unprocessable_entity, error_key) unless params.with_indifferent_access.dig(:data, :attributes, field)
          end
          return render_json_error(:unprocessable_entity, :invalid_status) unless Hermes::Statuses::HousingApplication.all_possible_status.include?(params.with_indifferent_access.dig(:data, :attributes, :status))

          @instance = find_application_to_update_status
          return  @instance unless @instance.is_a?(Hermes::HousingApplication)
          offer = @instance.housing_offer
          return render_json_error(:unprocessable_entity, :external_ref_not_matching_application) unless offer.external_ref == params.dig(:data, :attributes, :external_ref)

          json_reponse = {
            housing_application_id: @instance.id,
            housing_offer_id: offer.id,
          }

          data_attributes = params.with_indifferent_access.dig(:data, :attributes).symbolize_keys.slice(:status, :last_status_update_date, :date_event, :rank)

          if offer_has_valid_status(offer)
            if @instance.backward_status?(data_attributes[:status], @instance.status)
              json_reponse[:scenario] = "housing_application status is anterior - event sent to alin gestion"
              @instance.send_update_status(data_attributes)
            else 
              json_reponse[:scenario] = "housing_application updated - event sent to alin gestion"
              if Hermes::Statuses::HousingApplication.possible_status[:visit_status].include?(data_attributes[:status])
                params["data"]["attributes"]["visit_status"] = data_attributes[:status]
                params["data"]["attributes"].delete("status")
              end
              return respond_with_errors(@instance) unless @instance.update(mutate_valid_params)
            end
          else 
            json_reponse[:scenario] = "housing_offer statuses is end of life statuses - event sent to alin gestion"
            @instance.send_update_status(data_attributes)
          end

          render json: json_reponse
        end

        private

        def can_update_status
          params_status = params.dig(:data, :attributes, :status)
          allowed_status = ["EN ATTENTE", "ANNULEE", "DESISTEMENT AVANT CAL"]
          is_admin || allowed_status.include?(params_status) || params_status == @instance&.status && !@instance.new_record?
        end

        def find_application_to_update_status
          unique_number = params.dig(:data, :attributes, :unique_number)
          external_ref = params.dig(:data, :attributes, :external_ref)
        
          hr = Hermes::HousingRequest.find_by(
            is_active: true,
            :request_status.nin => ['ANNULEE', 'SATISFAITE'],
            unique_numbers: unique_number
          )

          return hr.active_application if hr&.active_application_id
        
          offer_ids = Hermes::HousingOffer.where(external_ref: external_ref).pluck(:id)
        
          if hr 
            ha = Hermes::HousingApplication.where(
              housing_request_id: hr.id,
              housing_offer_id: { '$in': offer_ids }
            ).order_by(created_at: :desc).first
            return ha if ha
            return render_json_error(:not_found, :housing_application_not_found)
          end
        
          hr_ids = Hermes::HousingRequest.where(unique_numbers: unique_number).pluck(:id)
        
          ha = Hermes::HousingApplication.where(
            housing_request_id: { '$in': hr_ids },
            housing_offer_id: { '$in': offer_ids }
          ).order_by(created_at: :desc).first
          return ha if ha
          return render_json_error(:not_found, :housing_request_and_application_not_found)
        end

        def offer_has_valid_status(offer)
          (offer.offer_status == "EN COURS D'ATTRIBUTION" && offer.lessor_status == "EN ATTRIBUTION") ||
          (offer.offer_status == "RETIRE PAR LE BAILLEUR" && offer.lessor_status == "REPRISE BAILLEUR A CONFIRMER") || 
          (offer.offer_status == "ARCHIVE" && offer.lessor_status == "RETIRE")
        end
        
      end
    end
  end
end
