# frozen_string_literal: true

class UpdateHousingOfferEligibleJob < ActiveJob::Base
  queue_as :x_hermes_eligibility
  unique :until_executed

  def perform(id)
    start_time = Time.now
    return unless (offer = Hermes::HousingOffer.find_by(id: id))

    old_eligibles_hash, old_eligibles_ids = get_old_eligibles(offer)
    new_eligibles_hash, new_eligibles_ids = get_new_eligibles(offer)
    bulk_ops = []

    old_eligibles_hash.each do |request_id, eligible|
      found_request = new_eligibles_hash[request_id]
      bulk_ops << if found_request
                    { update_one: { filter: { _id: eligible.first }, update: { '$set' => eligbible_values(offer, found_request) } } }
                  else
                    { delete_one: { filter: { _id: eligible.first } } }
                  end
    end

    (new_eligibles_ids - old_eligibles_ids).each do |request_id|
      found_request = new_eligibles_hash[request_id]
      bulk_ops << { insert_one: eligbible_values(offer, found_request) }
    end
    Hermes::Eligible.collection.bulk_write(bulk_ops)
  end

  private

  def get_old_eligibles(offer)
    old_eligibles = Hermes::Eligible.where(housing_offer_id: offer.id).pluck(
      :id,
      :housing_offer_id,
      :housing_request_id,
      :entity_id,
      :is_department_eligible,
      :is_seeked_eligible,
      :is_bordering_eligible
    )
    hash = old_eligibles.map { |a| [a[2], a] }.to_h
    ids = hash.keys

    [hash, ids]
  end

  def get_new_eligibles(offer)
    new_eligibles = Hermes::HousingRequest.where(offer.all_eligibility_query).hint('all_eligibility').pluck(
      :id,
      :seeked_departments,
      :seeked_district_insee_codes,
      :bordering_insee_codes,
      :entity_id
    )
    hash = new_eligibles.map { |a| [a.first, a] }.to_h
    ids = hash.keys

    [hash, ids]
  end

  def eligbible_values(offer, request_array)
    seeked_departments = request_array[1]
    seeked_district_insee_codes = request_array[2]
    bordering_insee_codes = request_array[3]
    entity_id = request_array[4]

    is_department_eligible = !!seeked_departments&.include?(offer.department),
    is_seeked_eligible = !!seeked_district_insee_codes&.include?(offer.insee_code)
    is_bordering_eligible = !!bordering_insee_codes&.include?(offer.insee_code)
    
    {
      housing_offer_id: offer.id,
      housing_request_id: request_array.first,
      entity_id: entity_id,
      is_department_eligible: is_department_eligible,
      is_seeked_eligible: is_seeked_eligible,
      is_bordering_eligible: is_bordering_eligible,
      is_seeked_bordering_eligible: is_seeked_eligible || is_bordering_eligible,
      offer_is_published: offer.published
    }
  end
end