class Hermes::EntityReservation
  include Mongoid::Document

  include Hermes::Concerns::CommonFieldsConcern
  include Hermes::Concerns::ActivableConcern
  field :entity_id, type: BSON::ObjectId
  field :housing_offer_id, type: BSON::ObjectId
  field :name, type: String
  field :from, type: Date
  field :to, type: Date
  field :last_is_active, type: DateTime

  common_indexes *%i[
    entity_id
    housing_offer_id
    name
    from
    to
    last_is_active
  ]

  belongs_to(:housing_offer, class_name: 'Hermes::HousingOffer')
  belongs_to(:entity, class_name: 'Hermes::Entity')
  
  validates_presence_of :entity_id, :housing_offer_id
  
  owner_field :user_id

  before_save :before_save_actions
  after_save :after_save_actions
  after_destroy :after_destroy_actions

  attr_accessor :skip_after_save

  def self.update_is_active
    today = Date.today
    not_active = Hermes::EntityReservation.unscoped.where(is_active: true).or(from: { '$gt': today }).or(to: { '$lt': today })
    new_active = Hermes::EntityReservation.unscoped.where(is_active: false, from: { '$lte': today }, to: { '$gte': today } )

    (not_active + new_active).each do |er|
      er.update_is_active
      er.save
    end
  end 

  def before_save_actions
    update_rights
    update_is_active
    update_last_is_active if is_active && is_active_changed?
    true
  end

  def update_rights
    self.user_id = entity.user_id if user_id.nil?
  end

  def update_is_active
    if from.nil? || to.nil?
      self.is_active = true
    else 
      self.is_active = (self.from.beginning_of_day .. self.to.end_of_day).cover?(DateTime.now)
    end 
  end

  def update_last_is_active
    self.last_is_active = DateTime.now
  end 
  
  def after_save_actions
    return if skip_after_save
    
    if is_active_changed?
      add_reservation_from_offer(housing_offer, entity_id) if is_active
      remove_reservation_from_offer(housing_offer, entity_id_was || entity_id) if !is_active && is_active_was
    elsif entity_id_changed?
      add_reservation_from_offer(housing_offer, entity_id) if entity_id
      remove_reservation_from_offer(housing_offer, entity_id_was) if entity_id_was
    end 
  end

  def add_reservation_from_offer(offer, e_id)
    offer.reload
    
    offer.update(active_reservations: (offer.active_reservations + [e_id.to_s]).uniq)
    create_status(true, e_id)
  end 

  def remove_reservation_from_offer(offer, e_id)
    offer.reload

    offer.update(active_reservations: (offer.active_reservations - [e_id.to_s]).uniq)
    create_status(false, e_id)
  end 

  def create_status(is_a, e_id)
    if entity = Hermes::Entity.find(e_id)
      Hermes::Status.create(
        reference_model_name: "HousingOffer",
        status_label: "entity_status",
        motive: is_a ? "Est entré en période de réservation" : "La période de réservation a prit fin",
        title: is_a ? "Réservé par #{entity.title}" : "Fin de réservation pour #{entity.title}",
        object_id: housing_offer_id,
        performer_id: e_id,
      )
    end 
  end 

  def after_destroy_actions
    return if skip_after_save

    remove_reservation_from_offer(housing_offer, entity.id)
  end 
end
