# frozen_string_literal: true

# Legende
# Les script a plusieurs parties indique que les bash (Batch dans l'app)
# peuvent s'initier avec differents types de source
# Le besoin est donc d'isoler la logique associee au modele actuel
# ne pas oublier d'indexer les champs simples

module Hermes
  # Housing offer model
  class HousingOffer
    include Hermes::Concerns::CommonFieldsConcern
    include Hermes::Concerns::AuditableConcern
    include Hermes::Concerns::AlsIdConcern
    include Hermes::Concerns::Status
    include Hermes::Concerns::ReservationHolder
    include ::Concerns::Geolocalizable
    include Hermes::Concerns::SopraXmlSender
    include Hermes::Concerns::EligibilityConcern
    
    include Mongoid::Locker
    index({locking_name: 1}, name: 'mongoid_locker_index', partial_filter_expression: {locking_name: {:$exists => true}}, expire_after_seconds: lock_timeout)
    
    hermes_tools 'typology'
    
    RENTED_STATUSES = %w[LOUE loué loue PLACE placé place].freeze
    PUBLISHED_STATUSES = ['LOGEMENT PUBLIE', 'PUBLIE'].freeze
    ATTRIBUTION_STATUSES = %w[requalification REQUALIFICATION]
    
    POSSIBLE_ELIGIBILITY_STATUSES = [
      "PUBLIE",
      "LOGEMENT PUBLIE",
      "EN COURS DE DESIGNATION",
      "EN COURS D'ATTRIBUTION",
      "RESERVE ALS",
      "EN ATTENTE COMMERCIALISATION",
      "COMMERCIALISE SANS PUBLICATION",
    ].freeze

    END_OF_LIFE_STATUSES = [
      "LOGEMENT PLACE",
      "RETIRE PAR LE BAILLEUR",
      "RENDU AU BAILLEUR",
      "RETIRE PAR ALS",
      "ARCHIVE",
    ]
    
    ORIENTATION_BOOLS = %i[
      priority_public
      single_parent
      mutation
    ].freeze
    
    
    field :requalification, type: Boolean, default: false
    field :dalo, type: Boolean
    field :entry_date, type: DateTime
    field :external_ref, type: String
    field :communicate_exact_address, type: Boolean
    field :typology, type: String
    field :type, type: String
    field :age, type: String
    field :department, type: String
    field :publication_end_date, type: DateTime
    field :max_applicants, type: Integer
    field :minimal_publication_days, type: Integer, default: 3
    field :expiry_days, type: Integer, default: 7
    field :rent_amount, type: Float
    field :rent_with_charges, type: Float
    field :surface, type: Integer
    field :rooms, type: Integer
    field :exclusivity_end_date, type: DateTime
    field :nb_of_eligible_employees, type: Integer
    field :nb_of_requests_from_employees, type: Integer
    field :availability_date, type: DateTime
    field :description, type: String
    field :details, type: String
    field :effort_rate, type: Float
    field :min_effort_treshold, type: Float
    field :single_parent, type: Boolean
    field :reserved, type: Boolean
    field :reservation_agreement_ref, type: String
    field :priority_public, type: Boolean
    field :mobility, type: Boolean
    field :bedrooms, type: Integer
    field :bathrooms, type: Integer
    field :floor, type: Integer
    field :floors, type: Integer
    field :elevators, type: Integer
    field :has_elevator, type: Boolean
    field :has_gardian, type: Boolean
    field :outside, type: String
    field :cellars, type: Integer
    field :has_cellar, type: Boolean
    field :parking_type, type: String
    field :parking_charges, type: Float, default: 0.0
    field :pmr, type: Boolean
    field :kitchen_type, type: String
    field :construction_year, type: Date
    field :address, type: String
    field :address_complements, type: String
    field :mutation, type: Boolean
    field :threshold_type, type: String
    field :threshold_subtype, type: String
    field :heating_type, type: String
    field :hot_water_distribution, type: String
    field :individual_heating, type: Boolean
    field :date_publication_start, type: DateTime
    field :date_publication_end, type: DateTime
    field :kind, type: String
    field :building_type, type: String
    field :already_rent, type: Boolean
    field :guarantee_deposit, type: Float
    field :rental_charges, type: Float
    field :reference, type: String
    field :release_date, type: Date
    field :dpe_ges, type: String
    field :dpe_conso, type: String
    
    field :ceil_category_1, type: Float
    field :ceil_category_2, type: Float
    field :ceil_category_3, type: Float
    field :ceil_category_4, type: Float
    field :ceil_category_5, type: Float
    field :ceil_category_6, type: Float
    field :ceil_category_7, type: Float
    field :ceil_category_8, type: Float
    field :ceil_category_9, type: Float
    field :ceil_category_10, type: Float
    field :ceil_category_11, type: Float
    field :ceil_category_12, type: Float
    field :floor_category_1, type: Float
    field :floor_category_2, type: Float
    field :floor_category_3, type: Float
    field :floor_category_4, type: Float
    field :floor_category_5, type: Float
    field :floor_category_6, type: Float
    field :floor_category_7, type: Float
    field :floor_category_8, type: Float
    field :floor_category_9, type: Float
    field :floor_category_10, type: Float
    field :floor_category_11, type: Float
    field :floor_category_12, type: Float
    field :min_revenues, type: Float
    field :total_charges, type: Float
    field :min_effort_rate, type: Float, default: 0.333
    field :tense_area, type: Boolean
    field :subsidiary_company, type: String
    field :rpls_id, type: String
    field :operation_name, type: String
    field :administrator, type: String
    field :administration_area, type: String
    field :potential_request_name_town, type: String
    field :potential_request_name_county, type: String
    field :number_of_companies_30_minutes_around, type: Integer
    field :construction_type, type: String
    field :address_can_be_shared, type: String
    field :lessor_code, type: String, default: "000"
    field :qpv, type: Boolean
    field :total_floors, type: String
    field :address_extended, type: String
    field :test_created_at, type: DateTime
    field :already_inhabited, type: Boolean
    field :new_offer, type: Boolean
    field :lessor_name, type: String
    field :lessor_siren, type: String
    field :als_siren, type: String
    field :postal_code, type: String
    field :insee_code, type: String
    field :residence_code, type: String
    field :residence_title, type: String
    # field :main_picture_id, type: BSON::ObjectId
    # field :pictures, type: String
    field :agency_id, type: BSON::ObjectId
    field :agency_name, type: String
    field :lessor_contact_name, type: String
    field :lessor_email, type: String
    field :lessor_phone_number, type: String
    field :terraces_or_balconies, type: Integer
    field :gardens, type: Integer
    field :has_garden, type: Boolean
    field :green_area, type: Boolean
    field :condo, type: Boolean
    field :furnished_lodging, type: Boolean
    field :virtual_visit, type: Boolean
    field :duplex, type: Boolean
    # TODO : verifier le match au noveau du WIKI
    field :first_quartile, type: String
    field :other_quartiles, type: String
    field :lessor_convention_reference, type: String
    field :is_public, type: Boolean
    field :published, type: Boolean
    field :accept_low_occupation, type: Boolean, default: false
    field :designation_date_end, type: DateTime
    field :designation_period_duration, type: Integer
    field :disability_accesses, type: Integer
    field :has_disability_access, type: Boolean, default: false
    field :eligible_requests_nb, type: Integer
    field :possible_eligible_requests_nb, type: Integer
    field :applicated_nb, type: Integer, default: 0
    field :application_following_contact, type: String
    field :entities_siret, type: Array
    field :eligible_requests_nb_detailed, type: Hash # of form { priority_public: 23, mutation: 45, ... }
    field :commercialized, type: Boolean, default: false
    field :entity_ids, type: Array, default: []
    field :active_reservations, type: Array, default: []
    field :active_reservations_rex_ids, type: Array, default: []
    # associated pictures
    field :ordered_picture_ids, type: Array, default: []
    field :picture_urls, type: Array, default: []
    field :is_mad, type: Boolean, default: false
    field :is_mad_updated_at, type: DateTime
    field :mad_als_date, type: DateTime
    
    field :last_update_by_lessor, type: DateTime
    field :auto_destroy_next_day, type: Boolean
    field :is_from_form, type: Boolean
    field :housing_channel_keys, type: Array, default: []
    field :reference_mad, type: String, default: ""
    field :lessor_orientations, type: Array, default: []

    field :reserving_siren, type: String
    field :reserving_business_name, type: String
    
    field :delai_reponse, type: Integer

    field :async_eligible_requests_nb, type: Integer
    field :async_eligible_department_nb, type: Integer
    field :async_eligible_seeked_nb, type: Integer
    field :async_eligible_bordering_nb, type: Integer
    field :async_eligible_seeked_bordering_nb, type: Integer
    field :async_eligible_other_from_department_nb, type: Integer 

    validates :lessor_siren, length: {is: 9}, allow_blank: true
    validates :als_siren, length: {is: 9}, allow_blank: true
    validates :reserving_siren, length: {is: 9}, allow_blank: true
    validate :reservation_holder_validation
    validate :ordered_picture_ids_length
    validate :complete_offer_validation

    validate :picture_urls_format
    validate :verifying_min_effort_rate_value
    alias_attribute :district, :city
    
    DEFAULT_ELIGIBILITY_INDEX = {
      single_parent: 1,
      priority_public: 1,
      mutation: 1,
      reserved: 1, 
      typology: 1, 
      min_revenues: 1, 
      housing_channel_keys: 1, 
      published: 1,
    }
    
    index(DEFAULT_ELIGIBILITY_INDEX, {name:"eligibility_base"})
    
    index({
      offer_status: 1,
      typology: 1, 
      min_revenues: -1, 
      reserved: 1, 
      single_parent: 1,
      priority_public: 1,
      mutation: 1,
      housing_channel_keys: 1,
    }, {name:"possible_eligibility"})

    common_indexes *%i[
      lessor_code
      entry_date
      external_ref
      publication_end_date
      rent_amount
      date_publication_start
      postal_code
      agency_id
      lessor_siren
      published
      lessor_id
      mutation
      mobility
      priority_public
      single_parent
      rent_with_charges
      is_public
      threshold_type
      threshold_subtype
      active_reservations
      ceil_category_1
      ceil_category_2
      ceil_category_3
      ceil_category_4
      ceil_category_5
      ceil_category_6
      floor_category_1
      floor_category_2
      floor_category_3
      floor_category_4
      floor_category_5
      floor_category_6
      reserving_siren
      department
      insee_code
      housing_channel_keys
      min_revenues
      typology
      reserved
      rpls_id
      async_eligible_requests_nb
      async_eligible_department_nb
      async_eligible_seeked_nb
      async_eligible_bordering_nb
      async_eligible_seeked_bordering_nb
      async_eligible_other_from_department_nb
    ]
    
    attr_accessor :will_update_lessor_objectives, :minimal_publication_date, :f5_sended
    
    belongs_to :threshold, class_name: 'Hermes::Threshold', index: true
    belongs_to :lessor, class_name: 'Hermes::Lessor', index: true
    belongs_to :reservation_holder, class_name: 'Hermes::ReservationHolder', index: true
    # TODO: handle also with_entity_ids=
    has_and_belongs_to_many :target_objectives, class_name: 'Hermes::LessorObjective'
    has_many :housing_entity_reservations, class_name: 'Hermes::EntityReservation'
    
    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id

    
    reader_auth_roles 'collaborateur offre logement'
    
    before_save :before_save_actions # https://dev.hermes.openstack.local/admin/families/5cffb7af4ac06e63bbd93298/mapping
    after_create :after_create_actions
    after_save :after_save_actions
    after_update :after_update_actions
    before_validation :before_validation_actions

    ELIGIBILITY_FIELDS = %w[
      offer_status
      is_public
      single_parent
      priority_public
      mutation
      mobility
      rent_with_charges
      min_revenues
      typology
      active_reservations
      housing_channel_keys
      department
      insee_code
      threshold_type
      threshold_subtype
    ].concat(
      (1..12).each_with_object([]) do |i, arr|
        %w[ceil floor].each {|type| arr << "#{type}_category_#{i}"}
      end
    ).freeze
    
    THRESHOLDS_FIELDS = %w[
      threshold_type
      threshold_subtype
    ].freeze
    
    HousingOffer.add_status(:single, 'lessor_status')
    HousingOffer.add_status(:single, 'offer_status')
    HousingOffer.add_status(:multiple, 'entity_status')
    
    scope :will_trigger_objective, ->(objective_id) {
      where(target_objective_ids: objective_id, :lessor_status.nin => RENTED_STATUSES, :offer_status.nin => ATTRIBUTION_STATUSES)
    }
    
    ###############################
    #### Notification #############
    include ::Concerns::NotificationCenter::Notifiable
    
    def self.targets_selector(housing_offer, opt = {})
      housing_offer.reload
      applicant_ids = opt.fetch(:target_ids, nil)
      eligibility = housing_offer.eligibility_query ? housing_offer.eligibility_query : {}
      applicant_ids ||= Hermes::HousingRequest.where(is_active: true, **eligibility).pluck(:main_applicant_id).map(&:to_s)
      Hermes::Applicant.where(:_id.in => applicant_ids || [])
    end

    def self.end_of_life_statuses
      END_OF_LIFE_STATUSES
    end 

    def self.eligible_request_estimate_min
      HERMES_CONFIG['eligible_request_estimate_min'] || 1000
    end 

    def self.calculate_estimate(number)
      number - (number % (HERMES_CONFIG['eligible_request_estimate_step'] || 1000))
    end 

    def self.calculate_eligibility_count_offer
      status = [
        "COMMERCIALISATION NON PUBLIEE",
        "COMMERCIALISE SANS PUBLICATION",
        "EN ATTENTE COMMERCIALISATION",
        "EN COURS ATTRIBUTION",
        "EN COURS D'ATTRIBUTION",
        "EN COURS DE DESIGNATION",
        "LOGEMENT PUBLIE",
      ]

      Hermes::Eligible.delete_all

      total = 0
      Hermes::HousingOffer.where(:active_reservations.nin => [nil, []], :offer_status.in => status).batch_size(5).no_timeout.each do |offer|
        begin
          UpdateHousingOfferEligibleJob.perform_now(offer.id.to_s)
          eco = Hermes::EligibleCountOffer.create(housing_offer: offer)
          puts "#{eco.housing_offer_id} - #{eco.created_at}; #{eco.offer_status}; #{eco.reserved_entitites_eligible_request_nb}"
          total += 1
        rescue => e
          puts "Error: #{e.message} - #{e.backtrace}\n"
        end
      end

      puts "Eligible count calculated on #{total} offers\n"

      eco_to_delete = Hermes::EligibleCountOffer.where(:created_at.lt => 7.day.ago)
      count = eco_to_delete.count

      eco_to_delete.destroy_all
      puts "Old eligible count deleted on #{count} offers\n"
    end 

    def geocode_if_need
      return geocode if address_is_blank?
    
      reverse_geocode if coordinates.blank?
    end
    
    def geocode_if_changed
      %w[address postal_code city country].each do |field|
        return reverse_geocode if send("#{field}_changed?") && field.present?
      end
    
      geocode if coordinates_changed? && coordinates.present? && address_is_blank?
    end
    
    def address_is_blank?
      address.blank? || postal_code.blank? || city.blank? || country.blank?
    end

    def find_housing_application_with_entity_id(entity_ids)
      housing_request_ids = Hermes::HousingRequest.where(is_active: true, :request_status.nin => ['ANNULEE', 'SATISFAITE'], :entity_id.in => entity_ids).pluck(:id)
      Hermes::HousingApplication.where(is_active: true, housing_offer_id: id, :housing_request_id.in => housing_request_ids)
    end 

    def notification_mailer
      ::Hermes::HermesMailer
    end
    
    def notifiable_title(key, opt = {})
      I18n.t("#{key}.title", reference: self.id)
    end
    
    def notifiable_body(key, opt = {})
      I18n.t("#{key}.body")
    end
    
    def udapte_siret_siren
      self.lessor_siren = self.lessor_siren&.slice(0, 9)
      self.als_siren = self.als_siren&.slice(0, 9)
    end
    
    def notifiable_path
      "https://als-hermes-int.be-idys.com/housing_offers/#{id}"
    end
    
    ###############################
    ###############################
    
    def compute_effort_rate
      typo_sym = typology&.upcase
      if HERMES_CONFIG["EFFORT_RATES"] && HERMES_CONFIG["EFFORT_RATES"][typo_sym].present?
        self.effort_rate = HERMES_CONFIG["EFFORT_RATES"][typo_sym]
      else
        self.effort_rate = (self.min_effort_rate || 0)
      end
    end
    
    def construction_year=(string)
      return super(nil) if string.blank?
      if string&.is_a?(String)
        return string.length == 4 ? super(Date.new(string.to_i)) : super(Date.parse(string))
      end 
      super(string) if string
    end
    
    def contact_group
      lessor&.contact_group
    end
    
    def main_contact
      contact_group&.main_contact
    end
    
    def entities
      Hermes::Entity.in(id: housing_entity_offers.pluck(:entity_id))
    end
    
    def agency
      Hermes::Entity.find_by(id: agency_id)
    end
    
    def history
      Hermes::LessorObjectiveHistory.where(housing_offer_id: self.id)
    end
    
    def inherit_lesssor_data
      return unless lessor
      
      self.lessor_name = lessor.title
    end
    
    def update_extras_presence
      %w[elevator garden cellar].each do |extra_name|
        send("has_#{extra_name}=", send(extra_name.pluralize)&.send('>', 0))
      end
    end

    def update_has_disability_access
      self.has_disability_access = disability_accesses == 3
    end 
    
    # set relationship with threshold
    def update_own_thresholds
      return unless threshold
      
      threshold.set_offer_thresholds(self)
    end
    
    def update_individual_heating
      self.individual_heating = heating_type && heating_type =~ /^Individuel/
    end
    
    def update_charges
      self.total_charges = (rental_charges.to_f + parking_charges.to_f + calculate_utility_charges).round(2)
    end
    
    def compute_without_charges
      # Nous supposons que si le loyer hors charges n'est pas indiqué, 
      # cela signifie que le montant des charges est nul 
      # et que le montant du loyer avec charges et le loyer hors charges sont égaux.
      self.rent_amount = rent_with_charges
    end
    
    def calculate_utility_charges
      return 0 if typology.blank? || heating_type.blank? || !['T1', 'T2', 'T3', 'T4', 'T5', 'T6'].include?(typology)
      
      non_electric = {
        'T1' => [31.59, 10.15],
        'T2' => [46.86, 15.06],
        'T3' => [62.84, 20.20],
        'T4' => [77.03, 24.76],
        'T5' => [104.58, 33.61],
        'T6' => [144.22, 45.71],
      }
      
      electric = {
        'T1' => [37.71, 11.38],
        'T2' => [55.09, 16.63],
        'T3' => [74.05, 22.29],
        'T4' => [90.40, 27.22],
        'T5' => [122.27, 36.81],
        'T6' => [166.29, 50.06],
      }
      
      if ['Individuel gaz', 'Individuel PAC Electrique', 'Collectif autre', 'Individuel autre'].include?(heating_type)
        return non_electric[typology].inject(:+)
      end
      
      if ['Individuel électrique'].include?(heating_type)
        return electric[typology].inject(:+)
      end
      0
    end
    
    def need_to_duplicate_offer
       (END_OF_LIFE_STATUSES.include?( offer_status) &&  lessor_status != "REPRISE BAILLEUR A CONFIRMER")
    end

    def update_min_revenues
      formatted_min_effort_rate = min_effort_rate.to_f
      total_rent = rent_with_charges.to_f + parking_charges.to_f
      self.min_revenues = if formatted_min_effort_rate.positive?
        ((1.0 / formatted_min_effort_rate) * total_rent).round(2)
      else
        0.0
      end
    end
    
    def update_threshold
      self.threshold = Hermes::Threshold.find_by("$or" => [{type: threshold_type}, {codes: threshold_type}])
    end
    
    def count_eligible_request_nb
      return 0 unless eligibility_query
      query = eligibility_query.merge(is_active: true, request_status: 'COMPLETE')
      Hermes::HousingRequest.where(query).count
    end 

    def count_possible_eligible_request_nb
      return 0 unless formated_eligibility
      query = formated_eligibility.merge(is_active: true)
      Hermes::HousingRequest.where(query).count
    end 

    def zero_eligible
      is_public || formated_eligibility.nil?
    end 
    
    def update_eligible_request_nb
      self.eligible_requests_nb = zero_eligible ? 0 : count_eligible_request_nb
      self.possible_eligible_requests_nb = zero_eligible ? 0 : count_possible_eligible_request_nb
    end

    def calculate_eligible_requests_nb!
      self.set(
        updated_at: DateTime.current, 
        eligible_requests_nb: zero_eligible ? 0 : count_eligible_request_nb,
        possible_eligible_requests_nb: zero_eligible ? 0 : count_possible_eligible_request_nb,
      )
    end
    
    def update_department
      code = insee_code || postal_code
      self.department = code&.length == 5 ? code[0..1].to_i > 95 ? code[0..2].to_s : code[0..1].to_s : ''
    end
    
    def update_is_mad_updated_at
      self.is_mad_updated_at = DateTime.current
      self.designation_date_end = (mad_als_date || is_mad_updated_at) + (delai_reponse || 30).days
    end

    def offer_settings
      setting = Hermes::HousingOfferExpirationSetting.find_by(department: department, lessor_id: lessor_id)
      setting = Hermes::HousingOfferExpirationSetting.find_by(department: department, lessor_id: nil) if setting.nil?
      setting
    end 

    def calculated_publication_end_date
      expiration = expiry_days
      expiration= offer_settings.expiry_days if offer_settings

      publication_end_date.presence || (date_publication_start + expiration)
    end 


    def handle_unpublished_status
      if PUBLISHED_STATUSES.include?(offer_status) && date_publication_start?
        if setting = offer_settings
          set(
            max_applicants: setting.max_applicants,
            minimal_publication_days: setting.minimal_publication_days,
            updated_at: DateTime.current
          )
        end
        
        @publication_end_date = calculated_publication_end_date
        
        if !max_applicants.nil? && max_applicants <= applicated_nb
          @publication_end_date = date_publication_start + minimal_publication_days
          if Time.zone.now > @publication_end_date
            return update(publication_end_date: Time.zone.now, offer_status: 'COMMERCIALISATION NON PUBLIEE', published: false)
          end
        end
        
        set(publication_end_date: @publication_end_date, updated_at: DateTime.current)
        UnpublishHousingOfferJob.set(wait_until: @publication_end_date + 1.days).perform_later(self.id.to_s, @publication_end_date.to_s)
      end
    end
    
    def reset_offers
      Hermes::HousingOffer.all.each_with_index do |off, i|
        lessor_siren1 = off.lessor_siren
        off.lessor_siren = nil
        off.save
        off.lessor_siren = lessor_siren1
        off.save
      end
    end
    
    def handle_is_mad
      return if is_mad == true
      
      self.is_mad = (offer_status == 'EN ATTENTE COMMERCIALISATION' && lessor_status == 'EN ATTENTE PUBLICATION')
      update_is_mad_updated_at if is_mad_changed?
    end

    def ordered_pictures
      Hermes::Picture.where(:id.in => ordered_picture_ids || []).sort_by { |pic| ordered_picture_ids&.map(&:to_s).index(pic.id.to_s) }
    end

    def picture_position(pictures, position)
      pictures.length > position - 1 ? pictures[position] : pictures.first 
    end 
    
    def main_picture(position=0)
      if ordered_pictures && !ordered_pictures.empty?
        picture_position(ordered_pictures, position)
      elsif picture_urls && !picture_urls.empty?
        picture_position(picture_urls, position)
      end
    end

    def picture_for_email(position = 0)
      return unless main_picture = main_picture(position)
      main_picture.is_a?(Hermes::Picture) ? main_picture.get_absolute_asset('thumb240') : main_picture[:thumb240]
    end 
    
    def soft_delete
      # "todo : soft delete of housing_offer"
      
      return if Rails.env == 'test'
      Hermes::BoClient::OfferStatusTask.new(housing_offer_id: id).execute
    end
    
    def set_log_action_history(action)
      Hermes::LogJob.add_to_queue(
        code: 'HERMES_HISTORY_LOG',
        action: "HousingOffer #{action}",
        housing_offer_id: self.id.to_s,
        current_user_id: Hermes::HousingOffer.current_user_id&.to_s
      )
      Hermes::HousingOffer.current_user_id = nil
    end
    
    def assign_target_objective
      # self.target_objective_ids=[]
      self.target_objectives = lessor.get_target_objectives_for(self)
    end
    
    def update_housing_channel_keys!(key, add = true)
      action = :delete
      if add
        raise Hermes::Errors::HousingChannelError, I18n.t('errors.housing_channel.already_added', key: key) if housing_channel_keys.include?(key)
        raise Hermes::Errors::HousingChannelError, I18n.t('errors.housing_channel.unavailable_for_departments', key: key, departments: department, count: 1) if !should_add_channel_key?(key)
        
        action = :push
      end
      housing_channel_keys.send(action, key)
      save!
    end
    
    def should_add_channel_key?(key)
      settings = Hermes::HousingChannelSetting.where(department: department)
      return settings.select {|setting| setting.housing_channel_keys.include? key}.any? if settings.any?
      
      true
    end
    
    def verifying_min_effort_rate_value
      return true if !min_effort_rate_changed?
      if min_effort_rate.blank? || min_effort_rate.to_f == 0.0
        errors.add :min_effort_rate, 'min_effort_rate do not equal to nil or 0'
      end 
    end  

    def before_save_actions
      picture_urls_format
      update_housing_channel_keys
      compute_effort_rate if effort_rate.nil?
      udapte_city if district_changed?
      udapte_siret_siren
      self.lessor_id = Hermes::Lessor.find_by(siren: lessor_siren)&.id if lessor_siren_changed?
      update_lessor_rights if lessor_id_changed?
      inherit_lesssor_data if lessor_id_changed?
      update_reservation_holder if reserving_siren_changed?
      update_extras_presence
      update_has_disability_access if disability_accesses_changed?
      update_department
      # update_department if postal_code_changed? || insee_code_changed? # fix temporaire
      update_individual_heating if heating_type_changed?

      if %i[rental_charges parking_charges typology heating_type].any? { |field| send("#{field}_changed?") }
        update_charges
      end
    
      if %i[min_effort_rate rent_with_charges parking_charges].any? { |field| send("#{field}_changed?") }
        update_min_revenues
      end

      update_threshold if threshold_type_changed?
      if threshold_id_changed? || threshold_subtype_changed?
        update_own_thresholds
      end
      
      if RENTED_STATUSES.include?(lessor_status)
        lessor_status = "LOUE"
      end
      # TODO: handle is_mad for status
      handle_is_mad if lessor_status_changed? || offer_status_changed?
      update_reserved
      update_eligibility_query if (changes.keys & ELIGIBILITY_FIELDS).length.positive?
      target_objective_computation_needed = (agency_id_changed? ||
        lessor_id_changed? ||
        operation_name_changed? ||
        district_changed? ||
        department_changed? ||
        threshold_type_changed? ||
        new_offer_changed? ||
        priority_public_changed? ||
        typology_changed?)
      assign_target_objective if (lessor && target_objective_computation_needed && (!lessor_status_changed? && !RENTED_STATUSES.include?(lessor_status)))
      
      handle_requalification_status
      if active_reservations_changed?
        update_reservations_readers 
        update_active_reservations_rex_ids
      end
    end

    def before_validation_actions
      compute_without_charges if rent_amount.blank?
    end 

    def update_reservation_holder
      self.reservation_holder_id = reserving_siren ? Hermes::ReservationHolder.find_by(siren: reserving_siren)&.id : nil
      self.reserving_business_name = reservation_holder&.name if reservation_holder_id_changed?
    end 
    
    def reservation_holder_validation
      return if !reserving_siren_changed? || !reserving_siren

      unless Hermes::ReservationHolder.find_by(siren: reserving_siren)
        errors.add :reservation_holder_id, 'no reservation holder found for this reserving_siren'
      end
    end 

    def ordered_picture_ids_length
      if ordered_picture_ids && ordered_picture_ids&.length > 20
        errors.add(:ordered_picture_ids, "cannot have more than 20 elements")
      end
    end 
    
    def complete_offer_validation
      return unless is_from_form && lessor_status == 'NON AFFECTE' && offer_status == 'EN ATTENTE COMMERCIALISATION'
  
      %i[
        external_ref
        typology
        rent_amount
        surface
        availability_date
        address
        postal_code
        threshold_type
        threshold_subtype
        kind
        rental_charges
        construction_type
        residence_title
        lessor_contact_name
        dpe_ges
        dpe_conso
        lessor_siren
      ].each do |field|
        errors.add(field, "can't be blank") if self[field].blank?
      end
    end

    def update_housing_channel_keys
      normal_keys = housing_channel_keys - ['township_attachment']
      self.housing_channel_keys =  
      if  housing_channel_keys == ['township_attachment']
        ['dc', 'township_attachment']
      elsif normal_keys.length != 1 && normal_keys.include?('dc')
        housing_channel_keys - ['dc']
      elsif housing_channel_keys.length == 0
        ['dc']
      else
        housing_channel_keys
      end 
    end 

    def update_reserved
      self.reserved = !self.active_reservations&.empty?
    end 

    def get_reader_group_id(entities)
      group_ids = []
      entities.each do |id|
        if entity = Hermes::Entity.find(id)
          group_ids += ['g_'+entity.group_id, 'g_'+entity.supervisor_group_id, 'g_'+entity.manager_group_id ]
        end
      end
      group_ids
    end 

    def update_reservations_readers
      new_entities = (active_reservations || []) - (active_reservations_was || [])
      removed_entities = (active_reservations_was || []) - (active_reservations || [])

      self.readers = (self.readers + get_reader_group_id(new_entities) - get_reader_group_id(removed_entities)).uniq
    end 
    
    def handle_requalification_status
      if ATTRIBUTION_STATUSES.include?(offer_status)
        self.requalification = true
      end
    end
    
    def udapte_city
      self.city = self.district
    end
    
    def update_lessor_rights
      s = lessor&.siren
      s_was = lessor_id_was ? Hermes::Lessor.find(lessor_id_was)&.siren : nil

      all_removed_entities = s_was ? Hermes::Entity.where(siren: s_was) : []
      all_added_entities = s ? Hermes::Entity.where(siren: s) : []
      
      all_removed_entities.each do |entity|
        self.readers.delete("g_#{entity.group_id}")
        self.writers.delete("g_#{entity.supervisor_group_id}")
        self.writers.delete("g_#{entity.manager_group_id}")
      end

      all_added_entities.each do |entity|
        self.readers = (self.readers || []).push("g_#{entity.group_id}").uniq
        self.writers = (self.writers || []).push("g_#{entity.supervisor_group_id}").uniq
        self.writers = (self.writers || []).push("g_#{entity.manager_group_id}").uniq
      end
    end
    
    def update_active_reservations_rex_ids
      self.active_reservations_rex_ids = [] if !self.active_reservations_rex_ids
      
      new_entities = (active_reservations || []) - (active_reservations_was || [])
      removed_entities = (active_reservations_was || []) - (active_reservations || [])
      
      new_entities.each do |entity_id|
        entity = Hermes::Entity.find(entity_id)
        self.active_reservations_rex_ids.push(entity.id_rex) if entity && !entity.id_rex.blank?
      end
      
      removed_entities.each do |entity_id|
        entity = Hermes::Entity.find(entity_id)
        if entity && !entity.id_rex.blank? && self.active_reservations_rex_ids.include?(entity.id_rex)
          self.active_reservations_rex_ids.delete(entity.id_rex)
        end
      end
    end
    
    def picture_urls_format
      if picture_urls_changed? && !picture_urls.blank?
        
        if picture_urls.is_a? Array
          picture_urls.each do |picture_url|
            if !( (picture_url.is_a? Hash) && (picture_url.key?("full_size")||picture_url.key?("thumb240") ||picture_url.key?("thumb1000")) )
              errors.add :picture_urls, "Ce champ doit etre renseigné avec un tableau d'objet du type : 
                [
                  { full_size: '......',
                    thumb240 : '........',
                    thumb1000 : '........'
                  },
                  {.....},
                  ...
                ]"
            end
          end
        else
          errors.add :picture_urls, "Ce champ doit etre renseigné avec un tableau d'objet du type : 
          [
            { full_size: '......',
              thumb240 : '........',
              thumb1000 : '........'
            },
            {.....},
            ...
          ]"
        end

      end 
    end
    def update_lessor_objectives
      lessor&.handle_objectives_for(self)
    end
    
    def destroy_tomorrow_if_no_update
      AutoDestroyOfferJob.set(wait: 36.hours).perform_later(id.to_s)
    end
    
    def after_create_actions
      set_log_action_history('created') if Hermes::HousingOffer.current_user_id
    end

    def after_save_actions
      check_status_changed
      update_lessor_objectives if lessor_status_changed? && RENTED_STATUSES.include?(lessor_status)
      
      update_lessor_objectives if lessor_status_changed? && RENTED_STATUSES.include?(lessor_status)
      destroy_tomorrow_if_no_update if auto_destroy_next_day && last_update_by_lessor_changed?

      soft_delete if offer_status_changed? && offer_status == "RETIRE PAR LE BAILLEUR"

      update_picture_offer if ordered_picture_ids_changed?
      if PUBLISHED_STATUSES.include?(offer_status)
        if !PUBLISHED_STATUSES.include?(offer_status_was) || (updated_at.year != updated_at_was&.year)
          Hermes::Accounting.create(year: Time.new.year, target: "offer", object_id: self.id)
        end
      end

      ExecuteFlux5TaskJob.set(wait: 1.minutes).perform_later(self.id.to_s) if is_mad && self.changed?
      
      if (changes.keys & ELIGIBILITY_FIELDS).length.positive?
        UpdateAsyncEligibleNbOfferJob.perform_later(self.id.to_s) 
      end 
      handle_unpublished_status
      
      set(published: PUBLISHED_STATUSES.include?(offer_status))
      
      run_flux_11_side_effects if need_flux_11_side_effects?
      run_flux_13_side_effects if need_flux_13_side_effects?
      update_readers_writers_application if readers_changed? || writers_changed?
    end
    
    def update_picture_offer
      added_pictures = (ordered_picture_ids || []) - (ordered_picture_ids_was || [])
      removed_pictures = (ordered_picture_ids_was || []) - (ordered_picture_ids || []) 

      added_pictures.each { |a| Hermes::Picture.find(a)&.update(housing_offer_id: self.id) }
      removed_pictures.each { |a| Hermes::Picture.find(a)&.update(housing_offer_id: nil) }
    end 

    def send_flux5_sync
      Hermes::LogJob.add_to_queue(code: "HERMES_FLUX5_SYNC", action: "housing_offer : #{self.id} -- Envoi du flux 5", data: self.attributes.slice(:_id, :als_id, :external_ref, :lessor_status, :offer_status, :lessor_status_id).transform_values{|v| v.to_s})
      res = ExecuteFlux5TaskJob.perform_now(self.id.to_s)
      if res
        Hermes::LogJob.add_to_queue(code: "HERMES_FLUX5_SYNC", action: "housing_offer : #{self.id} -- Flux 5 envoyé", data: res.attributes.transform_values{|v| v.to_s})
      else
        if xml_sopra_error_ids_changed? && xml_sopra_error_ids.any?
          error = Hermes::XmlSopraError.find(xml_sopra_error_ids.last)&.inspect
        else
          error = "Impossible d'identifier l'erreur, vérifier les codes de log BO_CLIENT_POST et BO_CLIENT_PUT pour l'offre en question"
        end
        Hermes::LogJob.add_to_queue(code: "HERMES_FLUX5_SYNC_ERROR", level: 'error', action: "housing_offer : #{self.id} -- Erreur d'envoi du flux 5", data: { "error" => error } )
      end
    end

    def update_readers_writers_application
      new_readers = readers_was.to_a - readers.to_a
      old_readers = readers.to_a - readers_was.to_a
      new_writers = writers_was.to_a - writers.to_a
      old_writers = writers.to_a - writers_was.to_a
      
      Hermes::HousingApplication.where(housing_offer_id: id).each do |app|
        app_readers = (app.readers.to_a + new_readers.to_a - old_readers.to_a).uniq
        app_writers = (app.writers.to_a + new_writers.to_a - old_writers.to_a).uniq
    
        app.set(updated_at: DateTime.current, readers: app_readers, writers: app_writers)
      end
    end
    
    def after_update_actions
      set_log_action_history('updated') if Hermes::HousingOffer.current_user_id
    end
    
    def update_lessor_on_all_offers
      offs = Hermes::HousingOffer.all
      counter = 0
      offs.each do |off|
        if (off.lessor_siren)
          off.lessor = Hermes::Lessor.find_by(siren: off.lessor_siren)
          off.save
          counter = counter + 1
        end
      end
    end

    def calculate_eligibility_base
      # boolean filters
      boolean_filters = %i[
        single_parent
        priority_public
        mutation
      ]
      h = attributes.select do |key, val|
        boolean_filters.include?(key.to_sym) && val
      end.each_with_object({}) do |(key, val), h|
        h[key.to_sym] = val unless val.nil?
      end
      
      # reserved filters reserve field => 'reserved'
      
      if reserved
        h[:entity_id] = {'$in' => active_reservations}
        h[:entity_validated] = true
      end
      
      # category filter
      h[:$or] = (1..12).map do |category|
        {
          category_type: category,
          reference_tax_revenues: {
            '$lte': get_tax_threshold(category),
            '$gte': get_tax_threshold(category, true)
          }
        }
      end
      
      # revenues filter
      h[:monthly_revenues] = {:$gte => min_revenues} if min_revenues.present?
      
      if housing_channel_keys.include?('township_attachment')
        h[:applicants_attached_codes] =  insee_code
      end 

      # housing_channels filter
      h[:housing_channel_keys] = {:$in => housing_channel_keys - ['township_attachment'] }
      
     
      if typology == "T6P"
        h[:max_typology] = { :$gte => 6 }
      else 
        typo_i = typology.blank? ?  nil : typology.gsub('T', '').gsub('P', '').to_i
        h[:min_typology] = { :$lte => typo_i }
        h[:max_typology] = { :$gte => typo_i }
      end 

      h[:is_active] = true
      h[:request_status] = 'COMPLETE'
      h
    end

    def calculate_eligibility_by_type(type)
      case type
      when 'department'
        {truly_seeked_departments: blank_or_missing(process_corsica(department))} 
      when 'seeked'
        {seeked_district_insee_codes: blank_or_missing(insee_code)} 
      when 'bordering'
        {bordering_insee_codes: blank_or_missing(insee_code)} 
      when 'seeked_bordering'
        {
          all_eligibility_codes: blank_or_missing(insee_code)
        } 
      when 'other_from_department'
        {
          truly_seeked_departments: blank_or_missing(process_corsica(department)),
          seeked_district_insee_codes: {:$ne => blank_or_missing(insee_code) },
          bordering_insee_codes: {:$ne => blank_or_missing(insee_code) },
        }
      end
    end

    def self.find_hint(type= 'department')
      mapping ={
        "department" => "department",
        "seeked" => "seeked",
        "bordering" => "bordering",
        "seeked_bordering" => "all_eligibility" ,
        "other_from_department" => "department",
        "all_eligibility" => "all_eligibility",
      }
      mapping[type]
    end 

    def all_eligibility_query
      query = eligibility_query_base.json_to_h
      return query.merge(all_eligibility_codes: 'missing value') if insee_code.blank?
      all_eligibility_codes = process_corsica(department)
      all_eligibility_codes = all_eligibility_codes[:$in] if all_eligibility_codes.is_a? Hash
      all_eligibility_codes = [all_eligibility_codes] if all_eligibility_codes.is_a? String
      all_eligibility_codes += [insee_code]
      query.merge(all_eligibility_codes: {"$in": all_eligibility_codes})
    end 

    def get_all_eligible_count(use_estimate = false)
      eligible_count = {}
      Hermes::HousingOffer.eligibility_types.each do |type|
        eligible_count["#{type}_count"] = if use_estimate && self["async_eligible_#{type}_nb"] && self["async_eligible_#{type}_nb"] > Hermes::HousingOffer.eligible_request_estimate_min
        requests = Hermes::HousingRequest.where(self.all_eligibility_query)
            Hermes::HousingOffer.calculate_estimate(self["async_eligible_#{type}_nb"])
          else
            eligibles_requests(type).count(hint: Hermes::HousingOffer.find_hint(type))
          end
      end
      eligible_count["total_count"] = if use_estimate && self["async_eligible_requests_nb"] && self["async_eligible_requests_nb"] > Hermes::HousingOffer.eligible_request_estimate_min
        requests = Hermes::HousingRequest.where(self.all_eligibility_query)
          Hermes::HousingOffer.calculate_estimate(self["async_eligible_requests_nb"])
        else
          eligible_count["seeked_bordering_count"] + eligible_count["other_from_department_count"]
        end 
      
      eligible_count
    end 

    def process_corsica(dep) 
      if ['2B', '2A', '20'].include? dep
        return  { :$in => ['2B', '2A', '20']}
      else
        return dep
      end
    end

    def eligibles_requests(type ='department')
      Hermes::HousingRequest.where(eligibility_query(type)).hint(Hermes::HousingOffer.find_hint(type))
    end 

    def is_eligible_request(hr, type ='department')
      eligibles_requests.pluck(:id).include?(hr.id)
    end 
    
    def get_tax_threshold(cat = nil, for_floor = false)
      prefixe = for_floor ? 'floor_category_' : 'ceil_category_'
      send("#{prefixe}#{cat}")
    end
    
    class << self
      def rented_statuses
        RENTED_STATUSES
      end
      
      def published_statuses
        PUBLISHED_STATUSES
      end
      
      def possible_eligibility_statuses
        POSSIBLE_ELIGIBILITY_STATUSES
      end
      
      def search_by_postal_codes(postal_codes)
        weights = postal_codes.uniq.reverse.each_with_index.map do |postal_code, idx|
          {'$cond' => [{'$eq' => ['$postal_code', postal_code]}, (idx + 1), 0]}
        end
        project = fields.keys.map do |field|
          {field => 1}
        end.reduce({}, :merge)
        
        collection.aggregate([
                               {'$project' => project.merge('weight': {'$add': weights})},
                               {'$sort' => {'weight': -1}}
                             ])
      end
    end
    
    def need_flux_11_side_effects?
      return unless lessor_status_changed? || offer_status_changed?
      
      lessor_status == 'RENDU AU BAILLEUR CONFIRME' && offer_status == 'RENDU AU BAILLEUR'
    end
    
    def run_flux_11_side_effects
      Flux11SideEffectsJob.perform_later(id.to_s)
    end
    
    def need_flux_13_side_effects?
      return unless lessor_status_changed? || offer_status_changed?

      (['RETIRE PAR LE BAILLEUR', 'RETIRE PAR ALS'].include?(offer_status) && lessor_status == 'RETIRE') ||
      (offer_status == 'ARCHIVE' && lessor_status == 'RETIRE') ||
      (offer_status == 'RETIRE PAR ALS' && lessor_status == 'RETIRE PAR ALS')
    end
    
    def run_flux_13_side_effects
      Flux13SideEffectsJob.perform_later(id.to_s)
    end
  end
end
