# frozen_string_literal: true

module Hermes
  # Dossier de candidature , gere le listing des pieces a fournir et le statut du dossier
  class HousingRequestFolder
    include Hermes::Concerns::CommonFieldsConcern
    include Hermes::Concerns::SopraXmlSender
    include Hermes::Concerns::ChangeMinusTwoYearConcern
    
    field :required_documents, type: Array, default: []
    field :required_document_ids, type: Array, default: []

    belongs_to :housing_request, class_name: 'Hermes::HousingRequest', index: true
    
    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id
    
    writer_auth_roles 'collaborateur entreprise', "collaborateur offre logement", 'superadministrateur'
    
    common_indexes *%i[
    ]
    
    before_create :before_create_action

    def main_applicant_id
      housing_request&.main_applicant_id.to_s
    end

    def is_untensed_area
      housing_request&.is_untensed_area
    end
        
    def get_year_label(delay)
      (get_minus_two_year.to_i + 2 - delay.to_i).to_s
    end

    def get_month_label(delay)
      date = (Date.current) + (delay * 31).send("days")
      month_labels = ['de Janvier', 'de Février', 'de Mars', "d'Avril", 'de Mai', 'de Juin', 'de Juillet', "d'Août", 'de Septembre', "d'Octobre", 'de Novembre', 'de Décembre']
      month_labels[date.month - 1] + " " + date.year.to_s
    end

    def get_types
      required_documents.map { |doc| doc[:type] }
    end 

    def types_include?(types)
      types = [types] unless types.kind_of?(Array)
      (types - get_types) == []
    end
    
    def get_labels
      labels = []
      required_documents.map do |doc|
        {
          type: doc[:type],
          applicant_id: doc[:applicant_id],
        }
      end
    end

    def calculate_required_documents
      self.required_documents = []

      [main_applicant_id, housing_request.applicant_ids].flatten.uniq.map(&:to_s).each do |applicant_id|
        calculate_identity_documents(applicant_id)
        calculate_ressources_documents(applicant_id)
        calculate_revenus_fiscaux(applicant_id)
        calculate_family_situation_documents(applicant_id)
      end

      calculate_actual_housing_documents
      calculate_motivation_documents
      calculate_disabilities_documents
      calculate_dependents_documents
    end

    private

    def calculate_identity_documents(applicant_id)
      return unless applicant = Applicant.find(applicant_id)
    
      is_main_applicant = applicant_id == main_applicant_id
      type = case applicant.nationality
             when '1'
              'Carte nationale d’identité ou passeport'
             when '3'
              'titre de séjour et Récépissé de renouvellement de titre de séjour si celui ci est expiré'
             when '2'
              "Carte d'identité d'un pays européen"
             end
    
      return unless type
    
      add_to_required_documents(
        nature: "Pièce d'identité",
        type: type,
        category: 'SPI',
        applicant_id: applicant_id
      )
    end
    
    def ressources_documents_rules
      [
        {
          monthly_incomes: %w[AUT],
          required_documents: [
            'Justificatif(s) autres ressources perçues',
          ]
        }, {
          monthly_incomes: %w[BE],
          required_documents: [
            "Carte d'étudiant délivrée par un établissement certifié et valable pour l'année en cours",
            "Avis d'attribution de bourse",
          ]
        }, {
          monthly_incomes: %w[RET],
          required_documents: [
            'Notification de pension',
          ]
        }, {
          monthly_incomes: %w[PINV],
          required_documents: [
            'Attestation de la Caisse d’allocations familiales (CAF) /Mutualité sociale agricole (MSA)',
          ]
        }, {
          monthly_incomes: %w[SAL],
          work_contract_type: %w[APP],
          required_documents: [
            "Contrat de travail de l'année en cours/ Bulletins de salaire du mois #{get_month_label(-1)}",
            "Contrat de travail de l'année en cours/ Bulletins de salaire du mois #{get_month_label(-2)}",
            "Contrat de travail de l'année en cours/ Bulletins de salaire du mois #{get_month_label(-3)}",
          ]
        }, {
          monthly_incomes: %w[SAL],
          work_contract_type: %w[AUT],
          required_documents: [
            'Justificatif(s) du salaire perçu',
          ]
        }, {
          monthly_incomes: %w[SAL],
          work_contract_type: %w[CDI CDD CDIP AGP],
          required_documents: [
            "Bulletin de salaire privé / public du mois #{get_month_label(-1)} ou Attestation de l'employeur",
            "Bulletin de salaire privé / public du mois #{get_month_label(-2)} ou Attestation de l'employeur",
            "Bulletin de salaire privé / public du mois #{get_month_label(-3)} ou Attestation de l'employeur",
          ]
        }, {
          monthly_incomes: %w[SAL],
          work_contract_type: %w[ART],
          required_documents: [
            "Dernier bilan ou attestation du comptable de l'entreprise évaluant le salaire mensuel perçu ou tout document comptable habituellement fourni à l'administration permettant de justifier les revenus",
          ]
    
        }, {
          monthly_incomes: %w[AMV],
          required_documents: [
            'Notification de pension de minimum vieillesse',
          ]
        }, {
          monthly_incomes: %w[PAR],
          required_documents: [
            'Extrait du jugement de divorce ou autre document démontrant la perception de la pension',
          ]
        }, {
          monthly_incomes: %w[CHO],
          required_documents: [
            "Avis de paiement de l'ARE",
          ]
        }, {
          monthly_incomes: %w[AF AAH AEEH AJPP PAJE PDA],
          required_documents: [
            'Attestation de la Caisse d’allocations familiales (CAF) /Mutualité sociale agricole (MSA)',
          ]
        }, {
          monthly_incomes: %w[RSA],
          required_documents: [
            'Attestation de la Caisse d’allocations familiales (CAF) /Mutualité sociale agricole (MSA)',
          ]
        }
      ]
    end
    
    def calculate_ressources_documents(applicant_id)
      return unless applicant = Applicant.find(applicant_id)
    
      applicant_monthly_charges =  applicant.monthly_charges && (applicant.monthly_charges&.[]('PAV')&.to_i || applicant.monthly_charges['PAV'.to_sym]&.to_i || 0)
    
      if applicant_monthly_charges && applicant_monthly_charges > 0
        add_to_required_documents(
          nature: 'Ressources',
          type: 'Extrait de jugement ou autre document démontrant le versement de la pension',
          category: 'SPR',
          applicant_id: applicant_id
        )
      end
    
      ressources_documents_rules.each do |rule|
        is_doc_needed = false
        work_contract_type_condition = true
        work_contract_type_condition = rule[:work_contract_type].include?(applicant.work_contract_type) if rule[:work_contract_type]
    
        rule[:monthly_incomes].each do |ressource_label|
          applicant_monthly_incomes = applicant.monthly_incomes && (applicant.monthly_incomes[ressource_label]&.to_i || applicant.monthly_incomes[ressource_label.to_sym]&.to_i || 0)
          next unless applicant_monthly_incomes && applicant_monthly_incomes > 0 && work_contract_type_condition
    
          is_doc_needed = true
        end
    
        next unless is_doc_needed
    
        rule[:required_documents].each do |type|
          add_to_required_documents(
            nature: 'Ressources',
            type: type,
            category: 'SPR',
            applicant_id: applicant_id
          )
        end
      end
    end
    
    def calculate_revenus_fiscaux(applicant_id)
      return unless applicant = Applicant.find(applicant_id)
    
      is_main_applicant = applicant_id == main_applicant_id
      coapp = nil
      coapp = housing_request.coapplicants.detect { |a| a && a[:id] == applicant_id.to_s } if !is_main_applicant && !housing_request.coapplicants.empty?
    
      not_need_rd = [
        'Conjoint',
        'Conjoint (e)',
        'M',
        'Pacsée',
        'P'
      ]
    
      if is_main_applicant || !applicant.dynamic_amount_minus_two.nil? || (coapp && !not_need_rd.include?(coapp[:rel_type]))
        add_to_required_documents(
          nature: "Avis d'imposition",
          type: "Avis d'imposition #{get_year_label(1)} sur les revenus de l'année #{get_year_label(2)}",
          category: 'RFR',
          applicant_id: applicant_id
        )
      end
    
      return unless applicant&.dynamic_amount_minus_one && (applicant&.dynamic_amount_minus_one&.>= 0)
    
      add_to_required_documents(
        nature: "Avis d'imposition",
        type: "Avis d'imposition #{get_year_label(0)} sur les revenus de l'année #{get_year_label(1)}",
        category: 'RFR',
        applicant_id: applicant_id
      )
    end
    
    def calculate_family_situation_documents(applicant_id)
      return unless applicant = Applicant.find(applicant_id)

      is_main_applicant = applicant_id == main_applicant_id

      rules = [
        {
          situation: 'P',
          type: "Attestation d'enregistrement de PACS",
        }, {
          situation: 'M',
          type: 'Livret de famille ou tout document équivalant démontrant le mariage',
        }, {
          situation: 'D',
          type: 'Jugement de divorce ou convention homologuée en cas de divorce par consentement mutuel',
        }, {
          situation: 'S',
          type: "Jugement de divorce ou convention homologuée en cas de divorce par consentement mutuel \nou\nMention de dissolution du pacs dans l'acte de naissance\nou\nSéparation d'un couple pacsé récépissé d'enregistrement de la déclaration\nou\nOrdonnance de non-conciliation ou, à défaut, copie de l’acte de saisine du juge aux affaires familiales dans les conditions prévues au code de procédure civile ou, lorsque c’est un divorce par consentement mutuel, justificatif de l’avocat du demandeur",
        }, {
          situation: 'V',
          type: 'Certificat de décès ou livret de famille',
        }
      ]
    
      rules.each do |rule|
        next if rule[:situation] != applicant.family_situation
    
        add_to_required_documents(
          nature: 'Situation familiale',
          type: rule[:type],
          category: 'SF',
          applicant_id: applicant_id
        )
      end
    end

    def rules_actual_housing_documents
      [
        {
          actual_housing_detail: %w[POCC],
          required_documents: [
            'Acte de propriété',
            'Plan de financement',
          ]
        }, {
          actual_housing_detail: %w[RSLL HLF RHVS HSH HHSH HCDFCM CDEFCM RE REU APCT],
          required_documents: [
            'Attestation de la structure d’hébergement ou du gestionnaire du logement-foyer',
          ]
        }, {
          actual_housing_detail: %w[FONC],
          required_documents: [
            'Contrat de bail',
            'Quittance de loyer ou attestation du bailleur indiquant que vous êtes à jour de vos loyers et charges, ou tout moyen de preuve des paiements effectués',
          ]
        }, {
          actual_housing_detail: %w[HCPE CPE HCP CP],
          required_documents: [
            'Attestation de la personne qui héberge',
          ]
        }, {
          actual_housing_detail: %w[SD SAHFB CAMCAR HCAM HHOT HH HABMOB HHM LOGT],
          required_documents: [
            'Attestation d’un travailleur social, d’une association ou certificat de domiciliation',
          ]
        }, {
          actual_housing_detail: %w[HLM LLS LP LSLLP],
          required_documents: [
            'Contrat de bail',
            'Quittance de loyer ou attestation du bailleur indiquant que vous êtes à jour de vos loyers et charges, ou tout moyen de preuve des paiements effectués',
          ]
        }
      ]
    end 
    
    def calculate_actual_housing_documents
      return unless applicant = Applicant.find(main_applicant_id)
    
      rules_actual_housing_documents.each do |rule|
        next unless housing_request.housing_type_list.is_a?(Array) && (rule[:actual_housing_detail] & housing_request.housing_type_list).length > 0
    
        rule[:required_documents].each do |type|
          add_to_required_documents(
            nature: 'Logement Actuel',
            type: type,
            category: 'SPR',
            applicant_id: main_applicant_id
          )
        end
      end

      return unless housing_request.housing_type_list && housing_request.housing_type_list.include?('SLHT')

      add_to_required_documents(
        nature: 'Logement Actuel',
        type: "(Vous êtes sous locataire)\n(« Bail et dernière quittance de loyer ou attestation du bailleur indiquant que le locataire est à jour des loyers et charges ou tout moyen de preuve des paiements effectués. »\n(Vous êtes hébergé dans un logement à titre temporaire)\n« Attestation de la personne qui héberge »", 
        category: 'SPL',
        applicant_id: main_applicant_id
      )
    end
    
    def rules_motivation_documents
      [
        {
          motivation: '15',
          type: 'Attestation de dépôt de demande de regroupement familial',
        }, {
          motivation: '02',
          type: 'Toute pièce établissant de la démolition de votre logement actuel',
        }, {
          motivation: '07',
          type: 'Carte d’invalidité ou décision d’une commission administrative compétente (commissions des droits et de l’autonomie des personnes handicapées) ou d’un organisme de sécurité sociale',
        }, {
          motivation: '05',
          type: 'Commandement de payer ou assignation à comparaître ou jugement prononçant l’expulsion ou commandement de quitter les lieux',
        }, {
          motivation: '22',
          type: 'Plan d\'apurement de la dette ou démarches en cours attestées par un travailleur social ou une association ou autre document démontrant les difficultés',
        }, {
          motivation: '26',
          type: 'Agrément assistant(e) maternel(le)',
        }, {
          motivation: %w[06 29 30],
          type: "Décision du juge prise en application de l'article 257 du code civil attestant de la situation d'urgence ou ordonnance de protection délivrée par le juge aux affaires familiales ou récépissé de dépôt de plainte",
        }, {
          motivation: '04',
          type: 'Lettre de congé du propriétaire ou jugement prononçant la résiliation du bail',
        }, {
          motivation: '24',
          type: "Document établi par un service public, un travailleur social, un professionnel du bâtiment ou une association ayant pour objet l'insertion ou le logement ou copie du jugement d'un tribunal statuant sur l'indécence du logement ou attestation de la CAF/MSA ou autre document démontrant l'indécence du logement",
        }, {
          motivation: '25',
          type: "Logement insalubre ou dangereux :\nDocument établi par un service public, un travailleur social, un professionnel du bâtiment ou une association ayant pour objet l'insertion ou le logement, photos, jugement du tribunal, attestation de la CAF ou de la CMSA, copie d'une décision de l'administration (mise en demeure, arrêté préfectoral ou municipal d'insalubrité remédiable ou irrémédiable ou de péril, ou de fermeture administrative s'il s'agit d'un hôtel meublé) ou autre document démontrant l'insalubrité ou la dangerosité du logement ou de l'immeuble\nLogement impropre à l'habitation :\nDocument établi par un service public, un travailleur social, un professionnel du bâtiment ou une association ayant pour objet l'insertion ou le logement, photos, jugement du tribunal, arrêté préfectoral de mise en demeure de faire cesser l'occupation des lieux ou autre document démontrant le caractère impropre à l'habitation",
        },
        {
          motivation: '31',
          type: "Attestation du chef de corps communal, intercommunal ou départemental (pour les Sapeurs-Pompiers Volontaires)",
        },
        {
          motivation: '32',
          type: "Attestation du conseil départemental ou extrait d'une décision judiciaire vous permettant d'établir que vous bénéficiez ou avez bénéficié d'une mesure au titre de l'ASE (pour les publics ASE)",
        },
        {
          motivation: %w[19 28],
          type: "Pièce justifiant de la localisation de l’emploi actuel ou futur",
        }
      ]
    end
    
    def calculate_motivation_documents
      all_motivations = housing_request.motivations
    
      rules_motivation_documents.each do |rule|
        rule_motivation = rule[:motivation]
        rule_motivation = [rule[:motivation]] if rule_motivation.is_a?(String)
    
        next unless all_motivations && all_motivations.length > 0 && !(all_motivations & rule_motivation).empty?
    
        add_to_required_documents(
          nature: 'Motifs de la demande',
          type: rule[:type],
          category: 'MD',
          applicant_id: main_applicant_id
        )
      end
    end
    
    def calculate_dependents_documents
    
      if housing_request.unborn_children&.length && housing_request.unborn_children[0] && housing_request.unborn_children[0][:anticipated_birth] &&
         Date.parse(housing_request.unborn_children[0][:anticipated_birth]) - 196.days < Date.today &&
         Date.today < Date.parse(housing_request.unborn_children[0][:anticipated_birth])

         add_to_required_documents(
          nature: 'Dépendants',
          type: 'Certificat de grossesse attestant que la grossesse est supérieure à 12 semaines',
          category: 'PC',
          applicant_id: main_applicant_id
        )
      end
    
      return unless housing_request&.dependents && (housing_request&.dependents&.length&.> 0)
    
      housing_request.dependents.each do |dep|
        dep.to_h.symbolize_keys!
        if dep[:rel_type] == 'E' || dep[:coparentality] == 'G' || dep[:coparentality] == 'D'
          if dep[:birth_date] && (Date.parse(dep[:birth_date]) <= Date.today - 18.years)

            add_to_required_documents(
              nature: 'Dépendants',
              type: "Carte nationale d'identité ou passeport ou titre de séjour ainsi que le récépissé de renouvellement de titre de séjour si celui-ci est expiré",
              category: 'PC',
              applicant_id: main_applicant_id,
              dependent_first_name: dep[:first_name],
              dependent_last_name: dep[:last_name]
            )
          end

          add_to_required_documents(
            nature: 'Dépendants',
            type: "Livret de famille ou acte de naissance",
            category: 'PC',
            applicant_id: main_applicant_id,
            dependent_first_name: dep[:first_name],
            dependent_last_name: dep[:last_name]
          )
        end

        next unless dep[:rel_type] == 'P' || dep[:rel_type] == 'A'
    
        add_to_required_documents(
          nature: 'Dépendants',
          type: "Carte nationale d'identité ou passeport ou titre de séjour ainsi que le récépissé de renouvellement de titre de séjour si celui-ci est expiré",
          category: 'PC',
          applicant_id: main_applicant_id,
          dependent_first_name: dep[:first_name],
          dependent_last_name: dep[:last_name]
        )
      end
    end
    
    def calculate_disabilities_documents
      return unless housing_request&.disability_details && housing_request&.disability_details.is_a?(Array) && (housing_request&.disability_details&.length&.> 0)
    
      housing_request.disability_details.each_with_index do |_disability_details, i|
        add_to_required_documents(
          nature: 'Handicap',
          type: "Carte d’invalidité ou décision d’une commission administrative compétente (commissions des droits et de l’autonomie des personnes handicapées) ou d’un organisme de sécurité sociale",
          category: 'HANDI',
          applicant_id: main_applicant_id,
          disability_index: "Pour la personne handicapée n°#{i + 1}",
        )
      end
    end

    def add_to_required_documents(nature: nil, type: nil, category: nil, applicant_id: nil, dependent_first_name: nil, dependent_last_name: nil, disability_index: nil)
      return unless (applicant = Applicant.find(applicant_id))
    
      self.required_documents << {
        nature: nature,
        type: type,
        category: category,
        is_for_main_applicant: main_applicant_id == applicant_id,
        applicant_id: applicant_id,
        applicant_full_name: applicant.full_name,
        dependent_first_name: dependent_first_name,
        dependent_last_name: dependent_last_name,
        disability_index: disability_index,
      }
    end
   
    def before_create_action
      self.user_id = housing_request&.user_id
    end
    
    def notify_applicant
    end
  end
end