
require 'rails_helper'
require 'vcr_config'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

HERMES_CONFIG['identity_provider_key'] = 'als_hermes_salarie'

describe Hermes::Api::Dmo::EntitiesController, :type => :api do
  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers

  check_mutate_params(
    model: Hermes::Entity, 
    mutate_params: Hermes::Api::Dmo::EntitiesController.mutate_params, 
    factory: :entity
  )

  let!(:responsible) {create(:user, roles: ['Administrator'])}
  let!(:superadministrateur) {create(:user, auth_roles: ['superadministrateur'])}
  let!(:user) {create(:user)}
  let!(:identity_provider) {create(:identity_provider,
    key: 'als_hermes_salarie',
    identifier_rights_url: 'https://rec.client.almerys.com/authz/realms/ALS/protocol/openid-connect/token',
    identifier_client_id: 'ALS_HERMES_BACK',
    identifier_client_secret: '77ff390e-6cca-4f09-9840-0e799a47a0ac',
    identifier_subject_issuer: 'ALS',
    account_service_url: 'https://rec.client.almerys.com/auth/realms/ALS/protocol/openid-connect/token',
    account_service_client_id: 'ALS_HERMES_SERVICE_ACCOUNT',
    account_service_client_secret: '28733e0d-c363-44a2-9f85-ea7d4784a4ad',
    create_user_url: 'https://api-rcc.be-ys.com/als-back/rcc/v1/accounts',
    add_roles_user_url: 'https://api-rcc.almerys.com/iam/api/authorization/clients/ALS_HERMES_BACK/users/{login}/roles/{role}?realm=ALS',
    create_user_headers: {
      'Content-Type': 'application/json',
      'X-GeXRT-API-Key': '169c24b0-a763-40f7-877d-7f94967ffea4',
    }
  )}
    
  describe 'GET /entities with operators' do
    let!(:entity1) {create(:entity, siret: 'Super000000000', spent_on_completed_requests: 50)}
    let!(:entity3) {create(:entity, siret: 'Super000000000', spent_on_completed_requests: 150)}
    let!(:entity2) {create(:entity, siret: 'Auper000000001', spent_on_completed_requests: 150)}
    describe 'with $lt' do
      before :each do
        authenticated_header(request, responsible)
        get 'index', {spent_on_completed_requests: {'$lt': 100}}
      end
      it 'returns results' do
        expect(response.status).to eq(200)
        expect(json['meta']['pagination']['total_objects']).to eq(1)
      end
    end
    describe 'with $in' do
      before :each do
        authenticated_header(request, responsible)
        get 'index', {siret: {'$in': ['Auper000000001', 'flamand']}}
      end
      it 'returns results' do
        expect(response.status).to eq(200)
        expect(json['meta']['pagination']['total_objects']).to eq(1)
      end
    end
    describe 'with $ne' do
      before :each do
        authenticated_header(request, responsible)
        get 'index', {spent_on_completed_requests: {'$ne': 50}}
      end
      it 'returns sorted results' do
        expect(response.status).to eq(200)
        expect(json['meta']['pagination']['total_objects']).to eq(2)
      end
    end
    describe 'with $regex' do
      before :each do
        authenticated_header(request, responsible)
        get 'index', {siret: {'$regex': /^S/}}
      end
      it 'returns sorted results' do
        expect(response.status).to eq(200)
        expect(json['meta']['pagination']['total_objects']).to eq(2)
      end
    end
  end

  describe 'GET /entities/:id' do
    describe 'verif it work' do
      let!(:entity1) {create(:entity)}
      let!(:user_right) {create(:user_right, 
        user_id: responsible.id.to_s, 
        entity_id: entity1.id.to_s,
        type: 'entity',
        right: 'administrator',
      )}
      before :each do
        authenticated_header(request, responsible)
        get 'show', {id: entity1.id}
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data']['attributes']['current_user_right']['right']).to eq('administrator')
      end
    end
    describe 'verif it work not work with a non authorized user' do
      let!(:entity1) {create(:entity)}

      before :each do
        authenticated_header(request, user)
        get 'show', {id: entity1.id}
      end
      it 'returns status code 404' do
        expect(response).to have_http_status(401)
      end
    end
  end

  describe 'GET /entities/:id/user_rights' do
    let!(:entity1) {create(:entity)}
    let!(:user1) {create(:user, email: 'user1@test.com', group_ids: [entity1.manager_group_id], auth_roles:['entreprise'])}
    let!(:user2) {create(:user, email: 'user2@test.com', group_ids: [entity1.supervisor_group_id], auth_roles:['bailleur'])}
    let!(:user3) {create(:user, email: 'user3@test.com', group_ids: [entity1.group_id], auth_roles:['cre'])}
    let!(:user4) {create(:user, email: 'user4@test.com', group_ids: [entity1.group_id], auth_roles:['rdsl'])}
    
    describe 'all user rights' do
      before :each do
        authenticated_header(request, responsible)
        get 'user_rights', {id: entity1.id}
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json.length).to eq(2)
      end
    end
    describe 'for specific user' do
      before :each do
        authenticated_header(request, responsible)
        get 'user_rights', {id: entity1.id, user_id: user2.id}
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['email']).to eq(user2.email)
        expect(json['right']).to eq('user')
      end
    end
  end

  describe 'GET /entities/:id/housing_requests_count' do
    let!(:entity) { create(:entity) }
    before :each do
      authenticated_header(request, responsible)
      get 'housing_requests_count', { id: entity.id }
    end

    it 'should return requests and delegations count' do
      expect(json['delegations']).to be_a Integer
      expect(json['in_process']).to be_a Integer
    end
  end

  context 'Superadmin searches' do
    let!(:superadmin) {create(:user, auth_roles: ['superadministrateur'], first_name: "test_first", last_name: "test_last")}
    let!(:user1) {create(:user, auth_roles: ['demandeur'], first_name: "John", last_name: "Doe")}
    let!(:user2) {create(:user, auth_roles: ['cre'], first_name: "Jane", last_name: "Doe")}
    let!(:entity1) {create(:entity, title: "test1", siret: "12345671234567",  siren: "123456789", id_cristal: "1")}
    let!(:entity2) {create(:entity, title: "test2", siret: "22345671234567",  siren: "223456789", id_cristal: "2")}
    let!(:entity3) {create(:entity, title: "test3", siret: "32345671234567",  siren: "323456789", id_cristal: "3")}
    let!(:user_right1) {create(:user_right, user_id: user1.id, entity_id: entity1.id,  type: 'entity', right: "reader")}
    let!(:user_right2) {create(:user_right, user_id: user2.id, entity_id: entity1.id,  type: 'entity', right: "administrator")}

    describe 'GET /entities/admin_search' do
      describe "search with user that is not a Superadmin" do
        before do
          authenticated_header(request, user)
          get 'admin_search'
        end
    
        it "refuses the access" do
          body = JSON.parse(response.body)
          expect(body["code"]).to eq(401)
        end
      end
      
      describe "search with good params" do

        before do
          authenticated_header(request, superadmin)
          get 'admin_search', {
            entity_params:  { siret: entity1.siret}
          }
        end
        
        it "as valid data" do
          datas = json['data']
          expect(datas.length).to eq(2)
          expect(datas[0]['entity_title']).to eq(entity1.title)
          expect(datas[0]['user_auth_roles']).to eq(user1.auth_roles)
        end
      end
      
      describe "search with good params + user params" do
        
        before do
          authenticated_header(request, superadmin)
          get 'admin_search', {
            entity_params: {title: "1"},
            user_params: {first_name: user2.first_name.downcase}
          }
        end
        
        it "as valid data" do
          datas = json['data']
          expect(datas.length).to eq(1)
          expect(datas[0]['entity_title']).to eq(entity1.title)
          expect(datas[0]['user_auth_roles']).to eq(user2.auth_roles)
        end
      end

      describe "search with id" do
        
        before do
          authenticated_header(request, superadmin)
          get 'admin_search', {
            entity_params: {id: entity1.id},
          }
        end
        
        it "as valid data" do
          datas = json['data']
          expect(datas.length).to eq(2)
          expect(datas[0]['entity_title']).to eq(entity1.title)
          expect(datas[0]['user_auth_roles']).to eq(user1.auth_roles)
        end
      end

      describe "when no matching" do 
        it 'return none' do 
          authenticated_header(request, superadmin)
          get 'admin_search', {
            entity_params: {title: "wrong title"},
          }
          datas = json['data']
          expect(datas.length).to eq(0)
        end 
      end 
    end 
  end 

  describe 'PUT /entities/:id' do
    let!(:entity) {create(:entity)}
    
    before :each do
      authenticated_header(request, responsible)
    end 

    it 'should update entity' do
      put 'update', {id: entity.id, data: {attributes: {siret: '12345678901234'}}}
      expect(response).to have_http_status(200)
      expect(json['data']['attributes']['siret']).to eq('12345678901234')
    end 

    it 'shoud send an error when empty params' do
      put 'update', {id: entity.id, data: {attributes: {wrong_attributes: '12345678901234'}}}
      expect(response).to have_http_status(422)
      expect(json['errors'].first['code']).to include('no_valid_parameters')
    end 
  end 
  
  describe 'PUT /entities/:id/switch_manual_mode' do
    let!(:entity) {create(:entity)}
    let!(:admin) { create :user, auth_roles: ['superadministrateur']}
    before do 
      authenticated_header(request, admin)

      put 'switch_manual_mode',{
        id: entity.id 
      }
    end 

    it 'does switch manual mode' do 
      resp = JSON.parse(response.body)
      expect(resp['code']).to eq(200)
    end 
  end 

  describe 'PUT /entities/:id/update_annual_stock' do
    let!(:entity) { create(:entity, annual_stock: 0) }
    let(:valid_params) { { id: entity.id, annual_stock: 2000 } }
    let(:invalid_id_params) { { id: 'invalid_id' } }
    let(:missing_params) { { id: entity.id } }
  
    context 'when user is super administrator' do
      before do
        authenticated_header(request, superadministrateur)
      end
  
      context 'with valid params' do
        it 'updates annual stock successfully' do
          put 'update_annual_stock', valid_params
          expect(response).to have_http_status(200)
          expect(json['data']['attributes']['annual_stock']).to eq(2000)
          expect(json['data']['attributes']['available_points']).to eq(2000)
        end
      end
  
      context 'with invalid entity id' do
        it 'returns not found' do
          put 'update_annual_stock', invalid_id_params
          expect(response).to have_http_status(404)
          expect(json['errors'].first['code']).to include('instance_not_found')
        end
      end
  
      context 'with missing params' do
        it 'returns unprocessable entity' do
          put 'update_annual_stock', missing_params
          expect(response).to have_http_status(422)
          expect(json['errors'].first['code']).to include('annual_stock_param_missing')
        end
      end
    end
  
    context 'when user is not super administrator' do
      before do
        authenticated_header(request, user)
      end
  
      it 'returns forbidden' do
        put 'update_annual_stock', valid_params
        expect(response).to have_http_status(403)
        expect(json['errors'].first['code']).to include('not_authorized')
      end
    end
  end

  describe 'GET /entities/search_sirens' do 
    10.times do |i|
      let!("entity#{i}".to_sym) {create(:entity, siren: "12300000#{i}")}
    end 
    let!(:other_entity) {create(:entity, siren: '123000000')}

    let!(:wrong_siren) {create(:entity, siren: '912345678')}

    before do
      authenticated_header(request, responsible)
    end 

    it 'works' do 
      get 'search_sirens', {siren: '123'}
      expect(response).to have_http_status(200)
      expect(json['data'].length).to eq(10)
      expect(json['data'].first['siren']).to eq(other_entity.siren)
      expect(json['data'].first['entities_nb']).to eq(2)
    end 

    it 'when not admin' do  
      authenticated_header(request, user)
      get 'search_sirens', {siren: '123'}
      expect(response).to have_http_status(401)
      expect(json['errors'].first['code']).to include('unauthorized')
    end

    it 'when missing siren' do 
      get 'search_sirens', {}
      expect(response).to have_http_status(422)
      expect(json['errors'].first['code']).to include('siren_missing')
    end 

    it 'when siren too short' do 
      get 'search_sirens', {siren: '12'}
      expect(response).to have_http_status(422)
      expect(json['errors'].first['code']).to include('siren_too_short')
    end 

    it 'with reverse order' do 
      get 'search_sirens', {siren: '123', sort: {siren: -1}}
      expect(json['data'].length).to eq(10)
      expect(json['data'].first['siren']).to eq('123000009')
      expect(json['data'].first['entities_nb']).to eq(1)
    end 

    it 'pagination' do 
      get 'search_sirens', {siren: '123', per_page: 3, page: 2}
      expect(json['data'].length).to eq(3)
      expect(json['meta']['pagination']['total_pages']).to eq(4)
      expect(json['meta']['pagination']['total_objects']).to eq(10)
      expect(json['data'].first['siren']).to eq('123000003')
    end 
  end 
  
end

describe Hermes::Api::Dmo::EntitiesController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  require_relative '../../swagger_helper'
  tags = 'API_DMO_ENTITIES'
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
  let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}
  let!(:superadmin) {create(:user, auth_roles: ['superadministrateur'])}
  let!(:admin) {create(:administrator)}

  path '/api/dmo/entities' do
    get 'Retrieves all entities' do
      tags tags
      produces 'application/json'

      Hermes::Api::Dmo::EntitiesController.acceptable_params.sort.each do |params_name, options|
        parameter name: params_name.to_s, in: options[:in] || :query, type: options[:type], required: options[:required] || false, description: options[:description] || ''
      end

      security [Bearer: {}]
      paginator

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }

        run_test!
      end 

      response '200', 'entities found' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        let!(:entities) {create_list(:entity, 5)}

        include_context 'with integration test'
        include_context 'valid jsonapi collection' do
          let(:expected_collection) {entities}
        end
      end
    end
  end

  path '/api/dmo/entities/admin_search' do
    get 'Get entities from searches params on entity and users ' do
      tags tags
      produces 'application/json'
      consumes 'application/json'

      security [Bearer: {}]

      parameter name: :params, in: :body, schema: {
        type: :object,
        properties: {
          entity_params: {
            type:  :object,
            properties: {
              id: {type: :string},
              id_cristal: {type: :string},
              siren: {type: :string},
              siret: {type: :string},
              title: {type: :string},
            }
          },
          user_params: {
            type:  :object,
            properties: {
              id: { type: :string},
              email: { type: :string},
              auth_roles: { type: :string},
              first_name: { type: :string},
              last_name: { type: :string},
            }
          },
        },
      }
      let(:params) {{}}

      response '401', 'unauthorized' do
        let(:Authorization) {"Bearer"}

        run_test!
      end

      response '200', 'entities found' do
        let(:Authorization) {"Bearer #{token_for(superadmin)}"}
        let!(:entity1) {create(:entity, title: "entity1", siren: "123456789")}
        let!(:entity2) {create(:entity, title: "entity2", siren: "987654321")}
        let!(:params) { {entity_params: {siren: entity1.siren} }}

        run_test!

        after do |_example|
          # test not working bc of a weird bug with rswag and params 
          # json = JSON.parse(response.body)
          # expect(json['data'].length).to eq(1)
          # expect(json['data'].first['id']).to eq(entity1.id.to_s)
        end

      end
    end
  end

  path '/api/dmo/entities/{id}' do
    get 'Retrieves a single entity by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        let(:id) { 1 }

        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { 1 }

        run_test!
      end 
      
      response '200', 'entity found' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        let(:entity) {create(:entity)}
        let(:id) {entity.id}

        include_context 'with integration test'
        include_context 'valid jsonapi ressource' do
          let(:expected_ressource) {entity}
        end
      end
    end
  end

  path '/api/dmo/entities/{id}/switch_manual_mode' do
    put 'switch manual mode' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]

      parameter name: :id, in: :path, type: :string

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        let(:id) { 1 }

        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(superadmin)}" }
        let(:id) { 1 }

        run_test!
      end 

      response '200', 'entity updated' do
        let(:Authorization) {"Bearer #{token_for(superadmin)}"}
        let(:entity) {create(:entity)}
        let(:id) {entity.id}

        include_context 'not containings errors'
        include_context 'with integration test' 
        after do |_example|
          json = JSON.parse(response.body)
          expect(json['code']).to eq(200)
        end
      end
    end
  end

  path '/api/dmo/entities/{id}/user_rights' do
    get 'Retrieves all user_rights on an entity' do
      tags tags
      produces 'application/json'

      parameter name: :id, in: :path, type: :string
      parameter name: :user_id, in: :query, type: :string
      let(:user_id) { nil }

      security [Bearer: {}]

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        let(:id) { 1 }

        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { 1 }

        run_test!
      end 

      response '200', 'user_rights found' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        let!(:entity) {create(:entity)}
        let!(:id) {entity.id}
        let!(:user_id)  { admin.id }
        let!(:user_rights) {create_list(:user_right, 1, type: 'entity', entity_id: entity.id, right: 'administrator', user_id: admin.id)}

        run_test!
      end
    end
  end

  path '/api/dmo/entities/{id}/housing_requests_count' do
    get 'Count request on an entity' do
      tags tags
      produces 'application/json'

      parameter name: :id, in: :path, type: :string

      security [Bearer: {}]

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        let(:id) { 1 }

        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { 1 }

        run_test!
      end 

      response '200', 'Count' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        let!(:entity) {create(:entity)}
        let!(:id) {entity.id}

        run_test!
      end
    end
  end

  path '/api/dmo/entities/{id}/update_annual_stock' do
    put 'Update annual stock' do
      tags tags
      produces 'application/json'


      parameter name: :id, in: :path, type: :string
      parameter name: :annual_stock, in: :query, type: :integer

      security [Bearer: {}]
      let!(:entity) {create(:entity)}
      let!(:id) {entity.id}
      let!(:annual_stock) { 2000 }
      let(:Authorization) { "Bearer #{token_for(admin)}" }

      response '404', 'not found' do
        let(:id) { 1 }

        run_test!
      end 

      response '422', 'missing annual_stock' do
        let!(:annual_stock) { nil }

        run_test!
      end 

      response '200', 'Updated annual stock' do
        run_test!
      end
    end
  end

  path '/api/dmo/entities/search_sirens' do
    get 'Search sirens' do
      tags tags
      produces 'application/json'

      parameter name: :siren, in: :query, type: :string, required: true

      security [Bearer: {}]
      let!(:entity) {create(:entity, siren: '123456789')}
      let(:Authorization) { "Bearer #{token_for(admin)}" }
      let!(:siren) { '123' }

      response '200', 'Found sirens' do
        run_test!
      end

    end
  end
end
