require 'rails_helper'
require [Hermes::Engine.root, %w[app lib notifications_center user]].flatten.join('/').to_s
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

def perform_job_now(job)
  job[:job].perform_now(*deserialize_argument(job[:args]))
end 

RSpec.describe NotificationsCenter::User, type: :helper do
  let!(:entity) { create(:entity, siret: '12345678901234', title: 'Compagny') }
  let!(:hr) { create(:complete_request) }
  let!(:offer1) { create(:housing_offer, insee_code: '75000', surface: 56, typology: 'T2', rent_with_charges: 840) }
  let!(:offer2) { create(:housing_offer, insee_code: '75000', surface: 32, typology: 'T1', rent_with_charges: 500) }

  before do 
    create_mail_template_default()
    HERMES_CONFIG["default_offer_alt_img_url"] = 'https://test.com/default_offer_alt_img_url.png'
    HERMES_CONFIG["new_reserved_offer_alt_img_url"] = 'https://test.com/new_reserved_offer_alt_img_url.png'
  end 

  shared_examples 'notification process' do |notification_key, expected_title, expected_body, expected_queue, job_class|
    let(:last_notif_job) { enqueued_jobs.select { |job| job[:job] == job_class }.last }
    let(:full_notif_key) { "user.#{notification_key}" }

    before do
      hr.main_applicant.update(main_employer_siret: entity.siret)
      hr.user.notify(
        key: NOTIFICATION_KEYS[:user][notification_key],
        channels: ['email'],
        datas: { offer_ids: [offer1.id.to_s, offer2.id.to_s] },
        is_slow: expected_queue == 'slow_notifications'
      )
    end

    it 'queues notification job' do
      expect(last_notif_job[:args][0]['key']).to eq(full_notif_key)
      expect(last_notif_job[:queue]).to eq(expected_queue)
    end

    it 'performs notification job and checks notification model' do
      perform_job_now(last_notif_job)
      notif = ::NotificationCenter::Notification.last
      expect(notif.key).to eq(full_notif_key)

      # Test content notification
      contents = ::NotificationsCenter::User.new(instance: notif)
      expect(contents.title).to eq(expected_title)
      expect(contents.body).to include(offer1.id.to_s, offer2.id.to_s)
      Array(expected_body).each do |body|
        expect(contents.body).to include(body)
      end 
    end
  end

  it_behaves_like 'notification process', 
    :new_reserved_offer, 
    "Action Logement Services AL'In Logement exclusif Compagny", 
    "new_reserved_offer_alt_img_url.png", 
    'slow_notifications', 
    SlowNotifierJob

  it_behaves_like 'notification process', 
    :new_published_offer, 
    "Action Logement AL'in_De nouvelles offres sont disponibles", 
    "default_offer_alt_img_url.png", 
    'slow_notifications', 
    SlowNotifierJob

  it_behaves_like 'notification process', 
    :application_not_selected,
    "Action Logement AL'in_Information sur votre candidature",
    [
      'Après étude de votre candidature sur le logement situé à',
      "default_offer_alt_img_url.png"
    ],
    'default',
    AsyncNotifierJob

  it_behaves_like 'notification process', 
    :application_offer_taken_back,
    "Action Logement AL'in_Information sur votre candidature",
    "default_offer_alt_img_url.png",
    'default',
    AsyncNotifierJob


  shared_examples 'handle email' do |notification_key|
    
    describe "#{notification_key}" do 
      let!(:test_email) { 'test@email.com' }
      
      subject do 
        enqueued_jobs.clear
        hr.user.notify(
          key: NOTIFICATION_KEYS[:user][notification_key],
          channels: ['email'],
          datas: { offer_ids: []}
        )
        last_job = enqueued_jobs.select { |job| job[:job] == AsyncNotifierJob }.last 
        perform_job_now(last_job)
      end
  
      it 'default email' do 
        subject
        notif = ::NotificationCenter::Notification.last
        contents = ::NotificationsCenter::User.new(instance: notif)
        expect(contents.email).to eq(hr.requester_email)
      end 
  
      it 'when no requester_email or applicant_used_for_siret_email' do 
        hr.set(requester_email: nil)
        subject
        notif = ::NotificationCenter::Notification.last
        contents = ::NotificationsCenter::User.new(instance: notif)
        expect(contents.email).to eq(hr.user.email)
      end 
  
      it 'when as applicant_used_for_siret_email' do 
        hr.set(applicant_used_for_siret_email: test_email)
        subject
        notif = ::NotificationCenter::Notification.last
        contents = ::NotificationsCenter::User.new(instance: notif)
        expect(contents.email).to eq(test_email)
      end 
    end 
  end

  it_behaves_like 'handle email', :new_reserved_offer
  it_behaves_like 'handle email', :entity_validation_accepted
  it_behaves_like 'handle email', :entity_validation_refused


  describe 'requests_to_approve' do 
    it 'when request' do
      enqueued_jobs.clear
      hr.update(requester_email: 'lolol.com')
      hr.user.notify(
        key: NOTIFICATION_KEYS[:user][:requests_to_approve],
        channels: ['email'],
        datas: {  number_of_requests: 3 }
      )
      last_job = enqueued_jobs.select { |job| job[:job] == AsyncNotifierJob }.last 
      perform_job_now(last_job)
      notif = ::NotificationCenter::Notification.last
      contents = ::NotificationsCenter::User.new(instance: notif)
      expect(contents.email).to eq(hr.user.email)
    end 
  end 

  describe 'full_name' do 
    let!(:hr) { create(:complete_request) }
    let!(:user) { hr.user }
  
    before do 
      user.set(first_name: 'User', last_name: 'Name')
      hr.set(
        requester_firstname: 'Requester', 
        requester_lastname: 'Name',
        applicant_used_for_siret_firstname: 'Usedforsiret',
        applicant_used_for_siret_lastname: 'Name'
      )
    end 
  
    shared_examples 'notification with full_name' do |notification_key, expected_name|
      it "returns correct full_name for #{notification_key}" do
        user.notify(
          key: NOTIFICATION_KEYS[:user][notification_key],
          channels: ['email'],
          datas: { offer_ids: [] }
        )
        last_job = enqueued_jobs.select { |job| job[:job] == AsyncNotifierJob }.last 
        perform_job_now(last_job)
        notif = ::NotificationCenter::Notification.last
        contents = ::NotificationsCenter::User.new(instance: notif)
        expect(contents.full_name).to eq(expected_name)
      end
    end
  
    %i[
      new_published_offer
      application_offer_taken_back
      application_not_selected
    ].each do |notification_key|
      include_examples 'notification with full_name', notification_key, 'Requester Name'
    end
  
    %i[
      new_reserved_offer
      entity_validation_accepted
      entity_validation_refused
    ].each do |notification_key|
      include_examples 'notification with full_name', notification_key, 'Usedforsiret Name'
    end

    include_examples 'notification with full_name', :other_notification, 'User Name'
  end
end
