require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

RSpec.describe Hermes::EligibleCountOffer, type: :model do
  it 'has a valid factory' do
    expect(FactoryBot.build(:eligible_count_offer)).to be_valid
  end

  describe 'calculate_count' do 
    let!(:offer) {create(:housing_offer, :with_eligible_requests, requests_count: 6)}
    let!(:eligible_count_offer) {create(:eligible_count_offer, housing_offer: offer)}
    let!(:entity1) {create(:entity)}
    let!(:entity2) {create(:entity)}
    let!(:entity3) {create(:entity)}

    before do
      offer.eligibles_requests.each_with_index do |request, i|
        request.set(entity_validated: true)
        request.set(entity_id: entity1.id) if [0, 1].include?(i)
        request.set(entity_id: entity2.id) if [2, 3, 4].include?(i)
        request.set(entity_id: entity3.id) if [5].include?(i)


        data = { seeked_departments: [offer.department], seeked_district_insee_codes: [], bordering_insee_codes: [] } if [0, 2].include?(i)
        data = { seeked_departments: [offer.department], seeked_district_insee_codes: [offer.insee_code], bordering_insee_codes: [] } if [1, 3, 4, 5].include?(i)

        request.set(data)
      end
    end 


    it 'when not reservation' do
      UpdateHousingOfferEligibleJob.perform_now(offer.id.to_s)

      expect(eligible_count_offer.reserved_entitites_eligible_request_nb).to eq({}) 
    end 

    it 'when reservation' do
      create(:entity_reservation, housing_offer: offer, entity_id: entity1.id, from: Date.today - 1.days, to: Date.today + 1.days)
      create(:entity_reservation, housing_offer: offer, entity_id: entity2.id, from: Date.today - 1.days, to: Date.today + 1.days)

      UpdateHousingOfferEligibleJob.perform_now(offer.id.to_s)

      expect(offer.active_reservations.count).to eq(2)

      eco = create(:eligible_count_offer, housing_offer: offer)

      eco.reload

      expect(eco.reserved_entitites_eligible_request_nb.keys).to eq([entity1.id.to_s, entity2.id.to_s]) 

      expect(eco.reserved_entitites_eligible_request_nb[entity1.id.to_s]['department']).to eq(2) 
      expect(eco.reserved_entitites_eligible_request_nb[entity1.id.to_s]['seeked_bordering']).to eq(1) 
      expect(eco.reserved_entitites_eligible_request_nb[entity1.id.to_s]['total']).to eq(2) 

      expect(eco.reserved_entitites_eligible_request_nb[entity2.id.to_s]['department']).to eq(3) 
      expect(eco.reserved_entitites_eligible_request_nb[entity2.id.to_s]['seeked_bordering']).to eq(2) 
      expect(eco.reserved_entitites_eligible_request_nb[entity2.id.to_s]['total']).to eq(3) 
    end 
  end 

  describe 'update_offer_status' do
    let!(:offer) {create(:housing_offer, offer_status: 'TEST')}
    let!(:eligible_count_offer) {create(:eligible_count_offer, housing_offer: offer)}

    it 'works' do 
      expect(eligible_count_offer.offer_status).to eq('TEST')
    end 
  end 
end
