require 'rails_helper'

RSpec.describe ::User, type: :model do
  describe 'Lessor association' do
    let!(:entity1) { create(:entity) }
    let!(:user_lessor) { create(:user, first_name: 'user_lessor', group_ids:[entity1.manager_group_id]) }
    let!(:user_lessor_associated) { create(:user) }
    let!(:user_not_associated) { create(:user) }
    let!(:lessor) { create(:lessor, user_id: user_lessor.id) }
    
    it 'can associate lessor and user' do
      user_lessor_associated.lessor = lessor
      user_lessor_associated.save
      user_lessor_associated.reload
      user_lessor.reload
      expect(user_lessor.lessor).to eq(lessor)
      expect(user_lessor_associated.lessor).to eq(lessor)
    end

    it 'should return true or false dependant of lessor association' do
      user_lessor.reload
      user_not_associated.reload
      expect(user_lessor.has_lessor?).to be(true)
      expect(user_not_associated.has_lessor?).to be(false)
    end

  end

  describe 'check_roles' do 
    let!(:user) { create(:user, auth_roles: ['salarie', 'entreprise']) }

    it 'remove entreprise' do 
      expect(user.auth_roles).to eq(['salarie', 'entreprise'])
    end

    it 'remove entreprise' do 
      user.update(auth_roles: ['salarie', 'entreprise', 'cre'])
      expect(user.auth_roles).to eq(['salarie', 'cre'])
    end

    it 'remove bailleur' do 
      user.update(auth_roles: ['salarie', 'bailleur', 'rdsl'])
      expect(user.auth_roles).to eq(['salarie', 'rdsl'])
    end

  end

  describe 'test group_ids' do 
    let!(:user) { create(:user)}
    let!(:entity) { create(:entity)}
    (1..100).each do |i|
      let!(:"entity_#{i}") { create(:entity)}
      let!(:"user_right_#{i}") { create(:user_right, user: user, entity: eval("entity_#{i}"), type: 'entity', right: 'administrator' )}
    end


    it 'works' do
      user.update(group_ids: user.group_ids + [entity.manager_group_id] - [entity_1.manager_group_id])
      expect(user.reload.group_ids.include?(entity.manager_group_id)).to eq(true)
      expect(user.reload.group_ids.include?(entity_1.manager_group_id)).to eq(false)
    end
  end 


  describe 'beiam' do 
    let(:user) { create(:user)}

    it 'use default provider' do 
      expect(HERMES_CONFIG['identity_provider_key']).to eq('als_hermes_salarie')
      expect(user.reload.provider).to eq('als_hermes_salarie')
    end 
  end 

  describe 'find_first_by_auth_conditions' do 
    let!(:user1) { create(:user, email: 'user1@test.com', login: 'user1')}
    let!(:user2) { create(:user, email: 'uSér2@test.com', login: 'User2')}

    it 'works' do 
      expect(User.find_first_by_auth_conditions(username: 'user1@test.com').id).to eq(user1.id)
      expect(User.find_first_by_auth_conditions(username: 'user1').id).to eq(user1.id)
      expect(User.find_first_by_auth_conditions(username: 'user2@test.com').id).to eq(user2.id)
      expect(User.find_first_by_auth_conditions(username: 'useR2').id).to eq(user2.id)
      expect(User.find_first_by_auth_conditions(username: 'lol')).to eq(nil)
    end
  end 

  describe 'userRight delete' do
    let!(:user) { create(:user)}
    let!(:entity) { create(:entity)}
    let!(:ur) {create(:user_right, user: user, type: 'entity', entity_id: entity.id, right: 'administrator')}

    it 'delete' do
      user.destroy
      expect(Hermes::UserRight.where(user_id: user.id).count).to eq(0)
    end
  end

  describe 'entity_lessor_with_user_right' do 
    let!(:user) { create(:user) }
    let!(:entity1) { create(:entity) }
    let!(:entity2) { create(:entity) }
    let!(:lessor1) { create(:lessor) }
    let!(:lessor2) { create(:lessor) }
  
    context 'when user has entity right' do
      before do
        create(:user_right, user: user, entity: entity1, type: 'entity', right: 'administrator')
      end
  
      it { expect(user.entity_lessor_with_user_right('entity').pluck(:id)).to eq([entity1.id]) }
    end
  
    context 'when user has lessor right' do
      before do
        create(:user_right, user: user, lessor: lessor1, type: 'lessor', right: 'reader')
      end
  
      it { expect(user.entity_lessor_with_user_right('lessor').pluck(:id)).to eq([lessor1.id]) }
    end
  end

  describe 'token_duration' do 
    let!(:user) { create(:user)}
    subject { User.decode_jwt_token(user.generate_jwt).first['exp'] }

    after do
      HERMES_CONFIG['token_duration'] = nil
    end

    it 'default' do 
      expect(subject).to eq((Time.now. + 4.hours).to_i)
    end

    it 'with config' do 
      HERMES_CONFIG['token_duration'] = 10
      expect(subject).to eq((Time.now. + 10.minutes).to_i)
    end 

    it 'with too big config' do 
      HERMES_CONFIG['token_duration'] = 10000
      expect(subject).to eq((Time.now. + 4.hours).to_i)
    end 

    it 'when expired' do 
      HERMES_CONFIG['token_duration'] = 10
      token = user.generate_jwt

      Timecop.freeze(Time.now + 11.minutes) do
        expect { User.decode_jwt_token(token) }.to raise_error(JWT::ExpiredSignature)
      end
    end 
  end 
  
  describe 'update_requests_to_approve_subscription' do 
    let!(:user) { create(:user)}

    subject { user.subscription.reload.keys.select { |k| k[:name] == 'requests_to_approve' }.first }

    it "doesn't subscribe to requests_to_approve" do
      expect(subject).to eq(nil)
    end 

    it 'when getting entreprise role' do
      user.update(auth_roles: ['entreprise'])
      expect(subject[:email]).to eq(true)
    end

    it 'dont lose subscription' do
      user.subscription.unsubscribe_to_email('requests_to_approve')
      user.reload.update(auth_roles: ['entreprise'])
      expect(subject[:email]).to eq(false)
    end
  end 
end
